import axios from 'axios';
import Cookies from 'universal-cookie';

const cookies = new Cookies();

export const vfAPI = axios.create( {
	baseURL: `${ localized.apiURL }/admin`,
	params: {
		apikey: localized.settings.general.api_key,
	},

} );

vfAPI.interceptors.request.use( ( config ) => {
	// Add tokens with each request, since the cookies may not be set or available when the Axios instance is created.
	const tokena = cookies.get( 'vendorfuel-admin-tokena' );
	const tokenb = cookies.get( 'vendorfuel-admin-tokenb' );

	if ( tokena && tokenb ) {
		config.params.tokena = tokena;
		config.params.tokenb = tokenb;
		return config;
	}
	// If tokens are missing, redirect to either login or billing page.
	if ( localized.settings.general.api_key ) {
		location.hash = '#!/login';
	} else {
		location.hash = '#!/subscribe';
	}
}, ( error ) => {
	return Promise.reject( error );
} );

vfAPI.interceptors.response.use( ( response ) => {
	if ( response.data?.errors && response.data?.warnings && response.data?.notifications ) {
		response.data.notices = []
			.concat( response.data.errors.map( ( error:string ) => {
				return { message: error, status: 'error' };
			} ) )
			.concat( response.data.warnings.map( ( warning: string ) => {
				return { message: warning, status: 'warning' };
			} ) )
			.concat( response.data.notifications.map( ( notification: string ) => {
				return { message: notification, status: 'info' };
			} ) );
	}
	return response;
}, ( error ) => {
	return Promise.reject( error );
} );
