declare const angular: ng.IAngularStatic;
import template from './orders-tracking.component.html';

export const OrderTrackingComponent: ng.IComponentOptions = {
	controller: OrderTrackingController,
	template,
};

function OrderTrackingController(
	$scope: any,
	Debug: any,
	Localized: any,
	Utils: any,
) {
	$scope.setUploading = setUploading;
	$scope.setFile = setFile;

	const vm = this;

	/**
	 * Initialize
	 */
	this.$onInit = () => {
		this.breadcrumbs = [
			{ name: 'Orders', state: 'orders.index' },
			{ name: 'Order Tracking', state: 'orders.tracking' },
		];
		getLog();
	};

	/**
	 */
	function getLog() {
		vm.isLoading = true;
		const endpoint = `${ Localized.apiURL }/admin/order/shipment/upload/log`;

		Utils.httpGet( endpoint )
			.then( function( response: any ) {
				if ( ! response.errors.length ) {
					vm.log = response.log;
				}
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				vm.isLoading = false;
			} );
	}

	/**
	 */
	function setUploading() {
		// eslint-disable-next-line no-shadow
		$scope.$apply( function() {
			vm.isUploading = ! vm.isUploading;
		} );
	}

	/**
	 * @param {Object} element DOM element
	 */
	function setFile( element: HTMLFormElement ) {
		// eslint-disable-next-line no-shadow
		$scope.$apply( function() {
			vm.theFile = element.files[ 0 ];
		} );
		const fd = new FormData();
		fd.append( 'file', vm.theFile );
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/shipment/upload/zip',
			data: fd,
			transformRequest: angular.identity,
			headers: {
				// eslint-disable-next-line no-undefined
				'Content-Type': undefined as undefined,
			},
		};
		Utils.getHttpPromise( req )
			.then( function() {
				angular.noop();
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				vm.isUploading = false;
			} );
	}
}

OrderTrackingController.$inject = [
	'$scope',
	'Debug',
	'Localized',
	'Utils',
];
