import React, { SyntheticEvent, useEffect, useState } from 'react';
import { Spinner } from '@wordpress/components';
import { vfAPI } from '../../core/vfAPI';
import { SearchBox } from '../../shared/SearchBox';
import { TableNav } from '../../shared/TableNav';
import { ListTable } from '../../shared/ListTable';
import { Notices } from '../../shared/Notices';

interface Params {
	page?: number;
	q?: string;
}

const apiUrl = '/purchasing/document-profile/';
const headers = [
	{ key: 'id', label: 'ID' },
	{ key: 'name', label: 'Name' },
	{ key: 'doc_format', label: 'Format' },
];

export const DocumentProfiles = () => {
	const [ isBusy, setIsBusy ] = useState<boolean>( false );
	const [ notices, setNotices ] = useState( [] );
	const [ page, setPage ] = useState<number>( 1 );
	const [ q, setQ ] = useState<string>( '' );
	const [ profiles, setProfiles ] = useState();
	const [ sortBy, setSortBy ] = useState<string>( 'id' );
	const [ sortType, setSortType ] = useState<'asc'|'desc'>( 'asc' );

	const deleteProfile = ( id: number ) => {
		setIsBusy( true );
		vfAPI.delete( `${ apiUrl }${ id }` )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					getProfiles();
				}
			} );
	};

	const editProfile = ( id: number ) => {
		const url = `${ location.href }/${ id }`;
		location.assign( url );
	};

	const getProfiles = ( params: Params = {} ) => {
		setIsBusy( true );
		const config = {
			params,
		};
		vfAPI.get( apiUrl, config )
			.then( ( response ) => {
				setProfiles( response.data.document_profiles );
				setIsBusy( false );
			} );
	};

	const handleSubmit = ( e: SyntheticEvent ) => {
		e.preventDefault();
		const params = {
			page: 1,
			q,
			sortBy,
			sortType,
		};
		getProfiles( params );
	};

	useEffect( () => {
		const params = {
			page,
			q,
			sortBy,
			sortType,
		};
		getProfiles( params );
	}, [ page, sortBy, sortType ] );

	return (
		<>
			<Notices notices={ notices } />
			<SearchBox isBusy={ isBusy } handleSubmit={ handleSubmit } q={ q } setQ={ setQ } />
			{ ! profiles && <div className="hstack justify-content-center">
				<Spinner />
			</div> }
			{ profiles &&
			<ListTable deleteItem={ deleteProfile } editItem={ editProfile } headers={ headers } isBusy={ isBusy } rows={ profiles.data } sortBy={ sortBy } setSortBy={ setSortBy } sortType={ sortType } setSortType={ setSortType } />
			}
			{ profiles &&
			<TableNav isBusy={ isBusy } currentPage={ profiles.current_page } setPage={ setPage } lastPage={ profiles.last_page } total={ profiles.total } />
			}
		</>
	);
};
