import React, { useEffect, useState } from 'react';
import { Spinner } from '@wordpress/components';
import { vfAPI } from '../core/vfAPI';
import { SearchBox } from '../shared/SearchBox';
import { TableNav } from '../shared/TableNav';
import { ListTable } from '../shared/ListTable';
import { Notices } from '../shared/Notices';

interface Params {
	page?: number;
	q?: string;
}

export const Roles = () => {
	const [ isBusy, setIsBusy ] = useState<boolean>( false );
	const [ notices, setNotices ] = useState( [] );
	const [ page, setPage ] = useState<number>( 1 );
	const [ q, setQ ] = useState<string>( '' );
	const [ roles, setRoles ] = useState();
	const [ sortBy, setSortBy ] = useState<string>( 'id' );
	const [ sortType, setSortType ] = useState<'asc'|'desc'>( 'asc' );

	const headers = [
		{ key: 'id', label: 'ID' },
		{ key: 'name', label: 'Name' },
		{ key: 'registration', label: 'Registration', type: 'boolean' },
		{ key: 'allowed_email_domains', label: 'Allowed Email Domains', type: 'array' },
	];

	const deleteRole = ( id: number ) => {
		setIsBusy( true );
		vfAPI.delete( `/customer/roles/${ id }` )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					getRoles();
				}
			} );
	};

	const editRole = ( id: number ) => {
		const url = `${ location.href }/${ id }`;
		location.assign( url );
	};

	const getRoles = ( params: Params = {} ) => {
		setIsBusy( true );
		const config = {
			params,
		};
		vfAPI.get( '/customer/roles', config )
			.then( ( response ) => {
				setRoles( response.data.roles );
				setIsBusy( false );
			} );
	};

	const handleSubmit = ( e: Event ) => {
		e.preventDefault();
		const params = {
			page: 1,
			q,
			sortBy,
			sortType,
		};
		getRoles( params );
	};

	useEffect( () => {
		const params = {
			page,
			q,
			sortBy,
			sortType,
		};
		getRoles( params );
	}, [ page, sortBy, sortType ] );

	return (
		<>
			<Notices notices={ notices } />
			<SearchBox isBusy={ isBusy } handleSubmit={ handleSubmit } q={ q } setQ={ setQ } />
			{ ! roles && <div className="hstack justify-content-center">
				<Spinner />
			</div> }
			{ roles &&
			<ListTable deleteItem={ deleteRole } editItem={ editRole } headers={ headers } isBusy={ isBusy } rows={ roles.data } sortBy={ sortBy } setSortBy={ setSortBy } sortType={ sortType } setSortType={ setSortType } />
			}
			{ roles &&
			<TableNav isBusy={ isBusy } currentPage={ roles.current_page } setPage={ setPage } lastPage={ roles.last_page } total={ roles.total } />
			}
		</>
	);
};
