import React, { useEffect, useState } from 'react';

interface Breadcrumb {
	name: string;
	hash: string;
}

interface Props {
	breadcrumbs: Breadcrumb[];
}

export const Breadcrumb = ( { breadcrumbs }: Props ) => {
	const hrefWithoutHash = location.href.replace( location.hash, '' );
	const [ last, setLast ] = useState<number>();

	useEffect( () => {
		setLast( breadcrumbs.length - 1 );
	}, [ breadcrumbs ] );

	return (
		<>
			{ breadcrumbs.length &&
				<nav aria-label="breadcrumb">
					<ol className="breadcrumb">
						<li className="breadcrumb-item">
							<a href="#!/dashboard">Dashboard</a>
						</li>
						{ breadcrumbs.map( ( breadcrumb: Breadcrumb, i: number ) =>
							<li key={ i } className={ `breadcrumb-item ${ i === last ? 'active' : '' }` }>
								{ i === last
									? <span>
										{ breadcrumb.name }
									</span>
									: <a href={ `${ hrefWithoutHash }${ breadcrumb.hash }` }>
										{ breadcrumb.name }
									</a>
								}
							</li> ) }
					</ol>
				</nav>
			}
		</>
	);
};
