import { highlightMenuItem } from './common/highlight-menu-item';
import { AccountingModule } from './admin/accounting/accounting.module';
import { ComponentsModule } from './components/components.module';
import { CommonModule } from './common/common.module';
import { DashboardModule } from './components/dashboard/dashboard.module';
import { EmailModule } from './admin/email/email.module';
import { OrdersModule } from './admin/orders/orders.module';
import { PurchasingModule } from './admin/purchasing/purchasing.module';
import { RolesModule } from './admin/roles/roles.module';
import { TaxesModule } from './components/taxes/taxes.module';
declare const angular: ng.IAngularStatic;
interface State {
	name: string;
	url: string;
	menuSlug: string;
	abstract?: boolean;
	component?: string;
	template?: string;
	onEnter?: any;
}

export const AppModule = angular
	.module( 'vendorfuelApp',
		[
			'ngAnimate',	// Required for UI Bootstrap
			'ngCookies',
			'ngResource',
			'ngSanitize',
			'ngTouch',		// Required for UI Bootstrap
			'ui.bootstrap',
			'ui.router',
			'ui.tinymce',
			'slickCarousel',
			'LocalStorageModule',
			'angular-toArrayFilter',
			AccountingModule,
			ComponentsModule,
			CommonModule,
			DashboardModule,
			EmailModule,
			OrdersModule,
			PurchasingModule,
			RolesModule,
			TaxesModule,
		] )
	.config( (
		$httpProvider: ng.IHttpProvider,
		$sceDelegateProvider: ng.ISCEDelegateProvider,
		$stateProvider: ng.ui.IStateProvider,
		$urlRouterProvider: ng.ui.IUrlRouterProvider,
		localStorageServiceProvider: ng.local.storage.ILocalStorageServiceProvider,
	) => {
		'ngInject';

		const templateDir = localized.dir.templates;
		$stateProvider
			.state( 'reports', {
				url: '/reports',
				templateUrl: templateDir + '/reports.html',
				controller: 'ReportsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'reports' );
				},
			} )
			.state( 'payments', {
				url: '/payments',
				templateUrl: templateDir + '/payments.html',
				controller: 'PaymentsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'payments' );
				},
			} )
			.state( 'subscribe', {
				url: '/subscribe',
				templateUrl: templateDir + '/subscribe.html',
				controller: 'SubscribeController as $ctrl',
				onEnter() {
					highlightMenuItem( 'subscription' );
				},
				params: {
					activeTab: {
						dynamic: true,
					},
				},
			} )
			.state( {
				name: 'login',
				url: '/login',
				templateUrl: templateDir + '/login.html',
				controller: 'LoginController as $ctrl',
			} )
			.state( 'reset-password', {
				url: '/reset-password?code&auth',
				templateUrl: templateDir + '/reset-password.html',
				controller: 'ResetPasswordController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
						code: null,
						auth: null,
					},
				},
			} )
			.state( 'billing-reset-password', {
				url: '/billing-reset-password?code&auth',
				templateUrl: templateDir + '/billing-reset-password.html',
				controller: 'BillingResetPasswordController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
						code: null,
						auth: null,
					},
				},
			} )
			.state( {
				name: 'logout',
				url: '/logout',
				component: 'vfLogout',
				onEnter() {
					highlightMenuItem( 'logout' );
				},
			} )
			.state( 'tenant-login', {
				url: '/tenant-login',
				templateUrl: templateDir + '/tenant-login.html',
				controller: 'TenantLoginController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
			} )
			.state( 'admin', {
				url: '/admin',
				templateUrl: templateDir + '/admin-accounts.html',
				controller: 'AdminAccountsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'admin' );
				},
			} )
			.state( 'catalog', {
				url: '/catalog',
				component: 'vfCatalog',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-products', {
				url: '/catalog-products/:activeTab',
				templateUrl: templateDir + '/catalog-products.html',
				controller: 'CatalogProductsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
					product: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-collections', {
				url: '/catalog-colllections/:activeTab',
				templateUrl: templateDir + '/catalog-collections.html',
				controller: 'CatalogCollectionsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-pricesheets', {
				url: '/catalog-pricesheets/:activeTab',
				templateUrl: templateDir + '/catalog-pricesheets.html',
				controller: 'CatalogPricesheetsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-categories', {
				url: '/catalog-categories/:activeTab',
				templateUrl: templateDir + '/catalog-categories.html',
				controller: 'CatalogCategoriesController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-promocodes', {
				url: '/catalog-promocodes/:activeTab',
				templateUrl: templateDir + '/catalog-promocodes.html',
				controller: 'CatalogPromoCodesController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-uom', {
				url: '/catalog-uom/:activeTab',
				templateUrl: templateDir + '/catalog-uom.html',
				controller: 'CatalogUomController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( 'catalog-banners', {
				url: '/catalog-banners/:activeTab',
				templateUrl: templateDir + '/catalog-banners.html',
				controller: 'CatalogBannersController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'catalog' );
				},
			} )
			.state( {
				name: 'customers',
				url: '/customers',
				abstract: true,
				onEnter() {
					highlightMenuItem( 'customers' );
				},
			} )
			.state( {
				name: 'customers.page',
				url: '',
				component: 'vfCustomers',
				onEnter() {
					highlightMenuItem( 'customers' );
				},
			} )
			.state( 'customers-groups', {
				url: '/customers-groups/:activeTab',
				templateUrl: templateDir + '/customers-groups.html',
				controller: 'GroupsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
					customer: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'customers' );
				},
			} )
			.state( 'customers-accounts', {
				url: '/customers-accounts/:activeTab',
				component: 'customerAccounts',
				params: {
					activeTab: {
						dynamic: true,
					},
					customer: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'customers' );
				},
			} )
			.state( 'shipping', {
				url: '/shipping',
				templateUrl: templateDir + '/shipping.html',
				controller: 'ShippingController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'shipping' );
				},
			} )
			.state( 'shipping-flat-rates', {
				url: '/shipping-flat-rates/:activeTab',
				templateUrl: templateDir + '/shipping-flat-rates.html',
				controller: 'ShippingFlatRatesController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'shipping' );
				},
			} )
			.state( 'shipping-parcels', {
				url: '/shipping-parcels/:activeTab',
				templateUrl: templateDir + '/shipping-parcels.html',
				controller: 'ShippingParcelsController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'shipping' );
				},
			} )
			.state( 'punchout', {
				url: '/punchout',
				component: 'vfPunchout',
				onEnter() {
					highlightMenuItem( 'punchout' );
				},
			} )
			.state( 'punchout-suppliers', {
				url: '/punchout-suppliers/:activeTab',
				templateUrl: templateDir + '/punchout-suppliers.html',
				controller: 'PunchoutSuppliersController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
					product: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'punchout' );
				},
			} )
			.state( 'tutorial', {
				url: '/tutorial',
				component: 'vfTutorial',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
			} )
			.state( 'billing', {
				url: '/billing',
				templateUrl: templateDir + '/billing.html',
				controller: 'BillingController as $ctrl',
				params: {
					activeTab: {
						dynamic: true,
					},
				},
				onEnter() {
					highlightMenuItem( 'billing' );
				},
			} );

		const states: Array<State> = [
			{
				name: 'manufacturers',
				url: '/manufacturers',
				abstract: true,
				template: '<ui-view/>',
				menuSlug: 'catalog',
			},
			{
				name: 'manufacturers.index',
				url: '',
				component: 'manufacturersIndex',
				menuSlug: 'catalog',
			},
			{
				name: 'manufacturers.create',
				url: '/create',
				component: 'manufacturersDetail',
				menuSlug: 'catalog',
			},
			{
				name: 'manufacturers.show',
				url: '/:id',
				component: 'manufacturersDetail',
				menuSlug: 'catalog',
			},
			{
				name: 'reviews',
				url: '/products/reviews',
				abstract: true,
				template: '<ui-view/>',
				menuSlug: 'catalog',
			},
			{
				name: 'reviews.index',
				url: '',
				component: 'reviewsIndex',
				menuSlug: 'catalog',
			},
			{
				name: 'reviews.show',
				url: '/:id',
				component: 'reviewsDetail',
				menuSlug: 'catalog',
			},
		];

		states.forEach( ( item ) => {
			const { menuSlug } = item;
			const state = item;
			state.onEnter = function() {
				highlightMenuItem( menuSlug );
			};

			$stateProvider.state( state );
		} );

		$httpProvider.interceptors.push( 'vfAdminInterceptor' );
		localStorageServiceProvider
			.setPrefix( 'vf' )
			.setStorageCookie( 0, '/', false )
			.setStorageType( 'sessionStorage' );

		$sceDelegateProvider.resourceUrlWhitelist( [
			// Allow same origin resource loads.
			'self',
			// Allow loading from our assets domain.  Notice the difference between * and **.
			'https://*.payfabric.com/**',
		] );
	} )
	.name;
