export class LocalizedService {
	'api_key': string;
	debugMode: boolean;
	'plugin_data': {
		Author: string;
		AuthorName: string;
		AuthorURI: string;
		Description: string;
		DomainPath: string;
		Name: string;
		Network: boolean;
		PluginURL: string;
		RequiresPHP: string;
		RequiresWP: string;
		TextDomain: string;
		Title: string;
		UpdateURI: string;
		Version: string;
	}
	notifications: { message: string, type: string}[];
	root: {
		url: string;
		path: string;
	}
	wpRestUrl: string;
	wpNonce: string;
	apiURL: string;

	constructor() {
		this.debugMode = localized.settings.general.debug;
		this.root = { url: localized.dir.url, path: localized.dir.root };
		this.apiURL = localized.apiURL;
		this.api_key = localized.settings.general.api_key;
		this.wpRestUrl = localized.dir.wpRestUrl + 'vendorfuel';
		this.wpNonce = localized.nonce;
		this.plugin_data = localized.plugin_data;

		this.notifications = [];
	}

	getNotifications() {
		return this.notifications;
	}

	setNotification( notification: {message: string, type: string} ) {
		const msg = notification.message;
		if ( msg && msg !== 'undefined' && msg !== null && msg !== '' && msg !== 'OK' ) {
			this.notifications.push( notification );
		}
	}

	setNotifications( data: { notifications: string[], warnings: string[], errors: string[]} ) {
		data.notifications.forEach( ( notification ) => {
			this.setNotification( {
				type: 'info',
				message: notification,
			} );
		} );

		data.warnings.forEach( ( warning ) => {
			this.setNotification( {
				type: 'warning',
				message: warning,
			} );
		} );

		data.errors.forEach( ( error ) => {
			this.setNotification( {
				type: 'error',
				message: error,
			} );
		} );
	}

	deleteNotification( index: number ) {
		if ( this.notifications[ index ] ) {
			this.notifications.splice( index, 1 );
		} else {
			this.notifications.push( {
				type: 'danger',
				message: 'Whoops! Unable to remove notification.',
			} );
		}
	}

	deleteNotifications() {
		this.notifications.splice( 0, this.notifications.length );
	}
}
