interface RootScopeWithBreadcrumbs extends ng.IRootScopeService {
	breadcrumbs: {
		name: string;
		state: string;
	}[]
}

/**
 * Catalog Page Component
 *
 * @namespace Components
 */
( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.component( 'vfCatalog', {
			controller: DashboardController,
			templateUrl: `${ localized.dir.templates }/catalog.component.html`,
		} );

	DashboardController.$inject = [
		'$rootScope',
		'Admin',
	];

	/**
	 * @param {Object} $rootScope AngularJS service
	 * @param {Object} Admin      VendorFuel service
	 */
	function DashboardController(
		$rootScope: RootScopeWithBreadcrumbs,
		Admin: any,
	) {
		const vm = this;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			this.breadcrumbs = [ {
				name: 'Catalog',
				state: 'catalog',
			} ];

			vm.isAuthed = Admin.Authed();
			vm.sections = [
				{
					title: 'Categories',
					state: 'catalog-categories({activeTab: 0})',
					icon: 'folder-fill',
					description: 'Manage product categories.',
					sublinks: [
						{
							title: 'All Categories',
							state: 'catalog-categories({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-categories({activeTab: 1})',
						},
						{
							title: 'Utilities',
							state: 'catalog-categories({activeTab: 3})',
						},
					],
				},
				{
					title: 'Products',
					state: 'catalog-products({activeTab: 0})',
					icon: 'box',
					description: 'Manage products.',
					sublinks: [
						{
							title: 'All Products',
							state: 'catalog-products({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-products({activeTab: 1})',
						},
						{
							title: 'Import',
							state: 'catalog-products({activeTab: 3})',
						},
						{
							title: 'Batches',
							state: 'catalog-products({activeTab: 4})',
						},
						{
							title: 'Utilities',
							state: 'catalog-products({activeTab: 6})',
						},
					],
				},
				{
					title: 'Reviews',
					state: 'reviews.index',
					icon: 'chat-fill',
					description: 'Manage product reviews.',
				},
				{
					title: 'Collections',
					state: 'catalog-collections({activeTab: 0})',
					icon: 'tag-fill',
					description: 'Manage product and category collections.',
					sublinks: [
						{
							title: 'All Collections',
							state: 'catalog-collections({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-collections({activeTab: 1})',
						},
					],
				},
				{
					title: 'Manufacturers',
					state: 'manufacturers.index',
					icon: 'building',
					description: 'Manage product manufacturers.',
					sublinks: [
						{
							title: 'All Manufacturers',
							state: 'manufacturers.index',
						},
						{
							title: 'Add New',
							state: 'manufacturers.create',
						},
					],
				},
				{
					title: 'Banners',
					state: 'catalog-banners({activeTab: 0})',
					icon: 'grid-fill',
					description: 'Manage banners.',
					sublinks: [
						{
							title: 'All Banners',
							state: 'catalog-banners({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-banners({activeTab: 1})',
						},
					],
				},
				{
					title: 'Price Sheets',
					state: 'catalog-pricesheets({activeTab: 0})',
					icon: 'cash-coin',
					description: 'Manage price sheets.',
					sublinks: [
						{
							title: 'All Price Sheets',
							state: 'catalog-pricesheets({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-pricesheets({activeTab: 1})',
						},
						{
							title: 'Import',
							state: 'catalog-pricesheets({activeTab: 3})',
						},
						{
							title: 'Batches',
							state: 'catalog-pricesheets({activeTab: 4})',
						},
					],
				},
				{
					title: 'Promo Codes',
					state: 'catalog-promocodes({activeTab: 0})',
					icon: 'wallet-fill',
					description: 'Manage promo codes.',
					sublinks: [
						{
							title: 'All Promo Codes',
							state: 'catalog-promocodes({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-promocodes({activeTab: 1})',
						},
					],
				},
				{
					title: 'Units of Measure',
					state: 'catalog-uom({activeTab: 0})',
					icon: 'rulers',
					description: 'Manage units of measure (UoM).',
					sublinks: [
						{
							title: 'All Units of Measure',
							state: 'catalog-uom({activeTab: 0})',
						},
						{
							title: 'Add New',
							state: 'catalog-uom({activeTab: 1})',
						},
					],
				},
			];
		};
	}
}() );
