import template from './customer-accounts.component.html';
declare const angular: ng.IAngularStatic;
declare const Papa: any;

interface Customer {
	group: any;
	password: string;
	'password_confirmation': string;
	'f1_name': string;
    'f1_replace_field': string;
    'f1_required': boolean;
    'f1_value': Array<string>;
    'f2_name': string;
    'f2_replace_field': string;
    'f2_required': boolean;
    'f2_value': Array<string>;
    'f3_name': string;
    'f3_required': boolean;
    'f3_value': Array<string>;
    'f4_name': string;
    'f4_required': boolean;
    'f4_value': Array<string>;
    'f5_name': string;
    'f5_required': boolean;
    'f5_value': Array<string>;
    'f6_name': string;
    'f6_required': boolean;
    'f6_value': Array<string>;
	pricesheet: unknown,
}

export const CustomerAccountsComponent: ng.IComponentOptions = {
	template,
	controller: CustomerAccountsController,
};

CustomerAccountsController.$inject = [
	'$scope',
	'$window',
	'$state',
	'$stateParams',
	'Admin',
	'Settings',
	'Debug',
	'Utils',
	'Localized',
	'SearchModal',
	'customerAccounts',
	'$timeout',
	'$uibModal',
];

function CustomerAccountsController(
	$scope: ng.IScope,
	$window: ng.IWindowService,
	$state: ng.ui.IStateService,
	$stateParams: ng.ui.IStateParamsService,
	Admin: any,
	Settings: any,
	Debug: any,
	Utils: any,
	Localized: any,
	SearchModal: any,
	customerAccounts: any,
	$timeout: ng.ITimeoutService,
	$uibModal: ng.ui.bootstrap.IModalService,
) {
	this.settings = Settings;

	this.$onInit = () => {
		this.breadcrumbs = [
			{ name: 'Customers', state: 'customers.page' },
			{ name: 'Accounts', state: 'customers-accounts' },
		];

		this.activeTab = parseInt( $stateParams.activeTab ) || 0;
		this.batches = [];
		this.batchEndpoint = Localized.apiURL + '/admin/customers/batch/';
		this.batchListOptions = {
			any: 'All',
			pending: 'Pending',
			confirmed: 'Confirmed',
			processing: 'Processing',
			processed: 'Processed',
		};
		this.customer = {
			pricesheet: {},
		} as Customer;
		this.customerData = customerAccounts.customerData();
		this.customerEndpoint = Localized.apiURL + '/admin/customers/';
		this.customers = {};
		this.isAuthed = Admin.Authed();
		this.isLoading = false;
		this.isLoadingMore = false;
		this.isShowingPassword = false;
		this.limitTypes = {
			none: 'None',
			monthly: 'Monthly',
			quarterly: 'Quarterly',
			flat: 'Flat',
		};
		this.isCostCenterRequired = localized.settings.general.checkout.cost_center_option_required;

		this.newCustomer = {
			pricesheet: {},
			promo_engine: true,
			punchout_suppliers: [],
			shipping_flat_rates: [],
			status: 'active',
		};
		this.passwordPattern = /(?=.*[0-9])(?=.*[!@#$%^&*-])[a-zA-Z0-9!@#$%^&*]{8,}/;
		this.replaceFields = {
			organization: 'Company/Organization',
			rr_po_num: 'Purchase Order Number',
			issuing_office: 'Issuing Office',
			cost_center_code: 'Cost Center Code',
			attention: 'Attention',
		};
		this.role = null;
		this.rppValues = [ 15, 30, 50, 100 ];
		this.searchOptions = {
			id: 'ID',
			name: 'Name',
			email: 'Email',
		};
		this.searchParams = {
			page: 1,
			q: '',
			searchBy: '',
			sortBy: 'id',
			sortType: 'asc',
			rpp: this.rppValues[ 0 ], // Must be defined after rppValues is declared.
		};
		this.searchTerm = '';
		this.selectedBatch = null;
		this.selectedCustomer = $stateParams.customer || null;
		this.sortAscending = true;
		this.sortBatchParams = {
			sort_by: 'batch_id',
			sort_type: '',
			status: 'any',
		};
		this.states = this.customerData.states;
		this.statuses = {
			active: 'Active',
			inactive: 'Inactive',
			unverified: 'Unverified',
		};
		this.updatedRates = [];

		//If the view tab is selected, redirect to the search page since selected product is null
		if ( this.activeTab === 2 && ! this.selectedCustomer ) {
			this.activeTab = 0;
		}
	};

	/**
	 * Cancels changing the user account password.
	 */
	this.cancelPasswordChange = () => {
		this.customer.password = null;
		this.customer.password_confirmation = null;
		this.newCustomer.password = null;
		this.newCustomer.password_confirmation = null;
		this.isSettingNewPassword = false;
	};

	/**
	 * Gets store settings used for default prefix
	 */
	this.getDefaultPrefix = () => {
		Settings.store.Get().finally( () => {
			this.isLoading = false;
		} );
	};

	/**
	 * Updates store settings user for default prefix
	 */
	this.updateDefaultPrefix = () => {
		Settings.store.Set().finally( () => {
			this.isLoading = false;
		} );
	};

	/**
	 * @param {number} i Tab index
	 */
	this.tabChanged = ( i: number ) => {
		this.activeTab = i;
		Settings.errors = {};
		this.isLoading = true;
		let req: {
				method: string;
				url: string;
				params?: any;
			} = {
				method: 'GET',
				url: this.customerEndpoint,
			};
		switch ( i ) {
			case 0:
				//First tab selected
				Utils.getHttpPromise( req )
					.then( ( resp: any ) => {
						this.priceSheets = resp.price_sheets;
						this.roles = resp.roles;
						this.customers = resp.customers;
						this.updatedRates = [];
					}, ( errResp: Error ) => {
						Debug.error( errResp );
					} ).finally( () => {
						this.isLoading = false;
					} );
				break;
			case 1:
				this.isLoading = false;
				break;
			case 2:
				req.url += this.selectedCustomer.id;
				Utils.getHttpPromise( req )
					.then( ( resp: any ) => {
						this.priceSheets = resp.price_sheets;
						this.roles = resp.roles;
						this.customer = this.sanitizeCustomer( resp.customer );
						this.suppliers = resp.suppliers;
						this.customer.shipping_flat_rates = [];
						this.updatedRates = [];
					}, ( errResp: Error ) => {
						Debug.error( errResp );
					} ).finally( () => {
						this.isLoading = false;
					} );
				break;
			case 3:
				this.isLoading = false;
				break;
			case 4:
				req = {
					method: 'GET',
					url: this.batchEndpoint,
					params: this.sortBatchParams,
				};
				Utils.getHttpPromise( req )
					.then( ( resp: any ) => {
						this.batches = resp.search_results;
						// eslint-disable-next-line no-shadow
						for ( let i = 0; i < this.batches.length; i++ ) {
							if ( this.batches[ i ].status === 'processing' ) {
								this.updateBatchProgress();
								break;
							}
						}
					}, ( errResp: Error ) => {
						Debug.error( errResp );
					} ).finally( () => {
						this.isLoading = false;
					} );
				break;
			case 5:
				req = {
					method: 'GET',
					url: this.batchEndpoint + this.selectedBatch.batch_id,
				};
				Utils.getHttpPromise( req )
					.then( ( resp: any ) => {
						this.selectedBatch = resp.batches[ 0 ];
						this.batchHeaders = resp.batches[ 0 ].columns;
					}, ( errResp: Error ) => {
						Debug.error( errResp );
					} ).finally( () => {
						this.isLoading = false;
					} );
				break;

			case 6:
				this.getDefaultPrefix();
				break;
		}
	};

	/**
	 * @param {number} num Number
	 */
	this.addCustomCheckoutValue = ( num: number ) => {
		if ( this.activeTab === 2 ) {
			this.customer[ 'f' + num + '_value' ].push( '' );
		} else if ( this.activeTab === 1 ) {
			if ( this.newCustomer[ 'f' + num + '_value' ] ) {
				this.newCustomer[ 'f' + num + '_value' ].push( '' );
			} else {
				this.newCustomer[ 'f' + num + '_value' ] = [];
				this.newCustomer[ 'f' + num + '_value' ].push( '' );
			}
		}
	};

	this.addCustomer = () => {
		this.isLoading = true;
		const req = {
			method: 'POST',
			url: this.customerEndpoint,
			data: this.newCustomer,
		};
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				if ( resp.errors.length <= 0 ) {
					this.selectedCustomer = resp.customer;
					this.newCustomer = {};
					this.activeTab = 2;
				}
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} ).finally( () => {
				this.isLoading = false;
			} );
	};

	this.changeTab = ( tabIndex: number, customerIndex: number ) => {
		this.activeTab = tabIndex;
		this.selectedCustomer = this.customers.data[ customerIndex ];
	};

	this.changeBatchTab = ( tabIndex: number, batchIndex: number ) => {
		this.activeTab = tabIndex;
		this.selectedBatch = this.batches[ batchIndex ];
	};

	this.checkPriceSheet = ( tabIndex: number ) => {
		if ( tabIndex === 1 ) {
			angular.forEach( this.priceSheets, ( sheet ) => {
				if ( this.newCustomer.price_sheet_id === sheet.price_sheet_id ) {
					this.selectedSheet = sheet.sheet;
				}
			} );
		}
		if ( tabIndex === 2 ) {
			angular.forEach( this.priceSheets, ( sheet ) => {
				if ( this.customer.price_sheet_id === sheet.price_sheet_id ) {
					this.selectedSheet = sheet.sheet;
				}
			} );
		}
	};

	this.confirmBatch = () => {
		this.isLoading = true;
		const req = {
			method: 'POST',
			url: this.batchEndpoint + 'confirm',
			data: {
				batch_id: this.selectedBatch.batch_id,
			},
		};
		Utils.getHttpPromise( req )
			.then( () => {
				this.selectedBatch = null;
				this.activeTab = 4;
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} ).finally( () => {
				this.isLoading = false;
			} );
	};

	this.formatDate = ( date: string ) => {
		const dateOut = new Date( date );
		return dateOut;
	};

	this.goToProduct = ( item: any ) => {
		$state.go( 'catalog-products', { activeTab: 2, product: item } );
	};

	/**
	 */
	this.openGroupSearchModal = () => {
		let callback = {};
		const data = {
			tabs: [
				{
					http: {
						method: 'GET',
						url: Localized.apiURL + '/admin/groups/',
						params: {
							q: '',
						},
					},
					relationships: [ 'groups' ],
					fields: [ 'group_id', 'name', 'parent_group_id', 'default_price_sheet' ],
					fieldPrefixes: [ 'ID: ', '', 'Parent ID: ', 'Price Sheet: ' ],
					id: 'group_id',
					selectOne: true,
				},
			],
		};
		callback = {
			confirm: this.searchResults,
			cancel: this.searchCancelled,
		};
		SearchModal.Show( callback, this.customer.name, data, 'Add items' );
	};

	this.onCopy = ( copiedCustomer: any ) => {
		this.customer = copiedCustomer;
	};

	this.openRatesSearchModal = () => {
		let callback = {};
		const config = {
			tabs: [
				{
					http: {
						method: 'GET',
						url: Localized.apiURL + '/admin/shipping/flat-rate/',
						params: {
							q: '',
							excludedField: 'customer_id',
							excludedId: this.selectedCustomer.id,
							excludedTable: 'shipping_flat_rate_customer',
						},
					},
					relationships: [ 'shipping_flat_rates' ],
					fields: [ 'name', 'cost', 'enabled' ],
					fieldPrefixes: [ 'Name: ', 'Cost: ', 'Enabled: ' ],
					id: 'id',
					selectOne: false,
				},
				{
					http: {
						method: 'GET',
						url: this.customerEndpoint + this.selectedCustomer.id,
					},
					relationships: [ 'customer', 'flatrates' ],
					fields: [ 'name', 'cost', 'enabled' ],
					fieldPrefixes: [ 'Name: ', 'Cost: ', 'Enabled: ' ],
					id: 'id',
					selectOne: false,
				},
			],
			updatedItems: this.updatedRates,
		};
		callback = {
			confirm: this.rateResults,
			cancel: this.searchCancelled,
		};
		//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
		SearchModal.Show( callback, 'Flat Rates', config, 'Add Shipping Rates' );
	};

	this.rateResults = ( resp: any ) => {
		if ( this.activeTab === 2 ) {
			this.updatedRates = [];
			resp.forEach( ( rate: any ) => {
				if ( rate.action === 'add' ) {
					this.updatedRates.push( rate );
					this.customer.shipping_flat_rates.push( { id: rate.value.id } );
				}
				if ( rate.action === 'unselect' ) {
					angular.forEach( this.customer.shipping_flat_rates, ( val, key ) => {
						if ( val.id === rate.value.id ) {
							this.customer.shipping_flat_rates.splice( key, 1 );
						}
					} );
				}
				if ( rate.action === 'remove' ) {
					this.updatedRates.push( rate );
					this.customer.shipping_flat_rates.push( { id: rate.value.id, deleted: 1 } );
				}
				if ( rate.action === 'deselect' ) {
					angular.forEach( this.customer.shipping_flat_rates, ( val, key ) => {
						if ( val.id === rate.value.id ) {
							this.customer.shipping_flat_rates.splice( key, 1 );
						}
					} );
				}
			} );
		}
	};

	this.rateCheck = ( rate: any ) => {
		let check = false;
		angular.forEach( this.updatedRates, ( key ) => {
			if ( key.value.id === rate.id && key.action === 'remove' ) {
				check = true;
			}
		} );
		return check;
	};

	this.removeCustomCheckoutValue = ( num: number, index: number ) => {
		if ( this.activeTab === 2 ) {
			this.customer[ 'f' + num + '_value' ].splice( index, 1 );
		} else if ( this.activeTab === 1 ) {
			this.newCustomer[ 'f' + num + '_value' ].splice( index, 1 );
		}
	};

	this.removeUploadFile = () => {
		this.accountSheetUpload = null;
		Localized.setNotification( {
			type: 'danger',
			message: 'File Successfully Removed',
		} );
	};

	this.sanitizeCustomer = ( customer: any ): Customer => {
		let hasMistypedValues = false;
		const sanitizedCustomer = customer;
		for ( let index = 1; index <= 6; index++ ) {
			const key = `f${ index }_value`;
			if ( typeof sanitizedCustomer[ key ] === 'string' || typeof sanitizedCustomer[ key ] === 'number' ) {
				sanitizedCustomer[ key ] = [ sanitizedCustomer[ key ] ];
				console.warn( `The '${ key }' property for ${ customer.name } ID #${ customer.id } has a mistyped value.` );
				hasMistypedValues = true;
			}
		}

		if ( hasMistypedValues ) {
			Localized.setNotification( {
				type: 'warning',
				message: 'One or more of the Custom Checkout Fields had a mistyped value that has been corrected. To review the changes, select the Checkout tab and click the Customize Checkout Fields button. To save the fixed changes, please click the Update button below.',
			} );
		}
		return sanitizedCustomer;
	};

	/**
	 * Callback that is used by the search modal
	 */
	this.searchCancelled = () => {
		angular.noop();
	};

	this.searchCustomers = () => {
		this.isLoadingMore = true;
		const req = {
			method: 'GET',
			url: this.customerEndpoint,
			params: this.searchParams,
		};
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				this.customers = resp.customers;
				this.roles = resp.roles;
				this.priceSheets = resp.price_sheets;
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} ).finally( () => {
				this.isLoadingMore = false;
			} );
	};

	/**
	 * Callback that is used by the search modal
	 *
	 * @param {Object} resp Response
	 */
	this.searchResults = ( resp: any ) => {
		if ( this.activeTab === 2 ) {
			this.customer.group = resp[ 0 ].value;
			this.customer.group_id = resp[ 0 ].value.group_id;
		} else if ( this.activeTab === 1 ) {
			this.newCustomer.group = resp[ 0 ].value;
			this.newCustomer.group_id = resp[ 0 ].value.group_id;
		}
	};

	this.setPage = ( page: number ) => {
		this.searchParams.page = page;
		this.searchCustomers();
	};

	this.setRpp = ( rpp: number ) => {
		this.searchParams.rpp = rpp;
		this.searchCustomers();
	};

	this.setQuery = ( query: string, searchBy: string ) => {
		this.searchParams.page = 1; // Reset page when query changes.
		this.searchParams.q = query;
		this.searchParams.searchBy = searchBy;
		this.searchTerm = this.searchParams.q;
		this.searchedBy = this.searchParams.searchBy;
		this.searchCustomers();
	};

	$scope.setUploading = () => {
		$scope.$apply( () => {
			this.uploading = ! this.uploading;
		} );
	};

	$scope.setFile = ( element: HTMLFormElement ) => {
		const vm = this;
		$scope.setUploading();
		$scope.$apply( () => {
			vm.theFile = element.files[ 0 ];
		} );
		const totalBytes = vm.theFile.size;
		if ( totalBytes < 1000000 ) {
			vm.fileSize = Math.floor( totalBytes / 1000 ) + 'KB';
		} else {
			vm.fileSize = Math.floor( totalBytes / 1000000 ) + 'MB';
		}
		const regex = /^([a-zA-Z0-9\s_\\.\-:])+(.csv|.txt)$/;
		if ( regex.test( vm.theFile.name.toLowerCase() ) ) {
			if ( typeof ( FileReader ) !== 'undefined' ) {
				const reader: any = new FileReader();
				reader.onload = () => {
					// eslint-disable-next-line no-undef
					Papa.parse( reader.result, {
						complete( results ) {
							const products: any[] = [];
							for ( let i = 0; i <= results.data.length; i++ ) {
								const info = results.data[ i ];
								const product: any[] = [];
								angular.forEach( info, ( val, key ) => {
									product[ key ] = val;
								} );
								products.push( product );
							}
							$scope.$apply( () => {
								vm.accountSheetUpload = products;
							} );
						},
					} );
					$scope.setUploading();
				};

				reader.readAsText( vm.theFile );
			} else {
				$window.alert( 'This browser does not support HTML5.' );
			}
		} else {
			vm.accountSheetUpload = null;
			vm.theFile = null;
			$scope.setUploading();
			$window.alert( 'Please upload a valid CSV file.' );
		}
	};

	/**
	 * @param {string} key Expecting 'shipping' or 'billing.
	 * @param {Object} e   Click event
	 */

	this.editAddresses = ( key: 'shipping' | 'billing', e: Event ) => {
		e.preventDefault();
		const customerId = this.customer.id;
		const modal = $uibModal.open( {
			component: 'customerAddressModal',
			resolve: {
				customerId() {
					return customerId;
				},
				key() {
					return key;
				},
			},
			size: 'lg',
		} );
		modal.result
			.then( () => {
				angular.noop();
			} );
	};

	/**
	 * @param {string} sortBy Sortby term
	 * @param {Object} e      Click event
	 */
	this.changeSortBy = ( sortBy: string, e: Event ) => {
		e.preventDefault();
		this.sortAscending = ( this.searchParams.sortBy === sortBy ) ? ! this.sortAscending : true;
		this.searchParams.sortBy = sortBy;
		this.searchParams.sortType = this.sortAscending ? 'asc' : 'desc';
		this.searchCustomers();
	};

	this.sortBatchIndex = ( sortBy: string ) => {
		this.sortAscending = ( this.sortBatchParams.sort_by === sortBy ) ? ! this.sortAscending : true;
		this.sortBatchParams.sort_by = sortBy;
		this.sortBatchParams.sort_type = this.sortAscending ? 'asc' : 'desc';
		this.sortBatches();
	};

	/**
	 */
	this.sortBatches = () => {
		this.isLoadingMore = true;
		const req = {
			method: 'GET',
			url: this.batchEndpoint,
			params: this.sortBatchParams,
		};
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				this.batches = resp.search_results;
				for ( let i = 0; i < this.batches.length; i++ ) {
					if ( this.batches[ i ].status === 'processing' ) {
						this.updateBatchProgress();
						break;
					}
				}
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} ).finally( () => {
				this.isLoadingMore = false;
			} );
	};

	/**
	 */
	this.updateCustomer = () => {
		this.isLoading = true;
		const endpoint = `${ this.customerEndpoint }${ this.customer.id }`;
		const data = this.customer;

		// Remove password property from data unless user is explicitly changing the password.
		if ( ! this.isSettingNewPassword && 'password' in data ) {
			delete data.password;
		}

		Utils.httpPut( endpoint, null, data )
			.then( ( resp: any ) => {
				if ( resp.errors.length <= 0 ) {
					angular.forEach( this.updatedRates, ( rate ) => {
						if ( rate.action === 'add' ) {
							this.customer.flatrates.push( rate.value );
						} else {
							angular.forEach( this.customer.flatrates, ( val, key ) => {
								if ( val.id === rate.value.id ) {
									this.customer.flatrates.splice( key, 1 );
								}
							} );
						}
					} );
					this.updatedRates = [];
				}
			} )
			.catch( ( errResp: Error ) => {
				Debug.error( errResp );
			} )
			.finally( (): void => {
				this.cancelPasswordChange();
				this.isLoading = false;
			} );
	};

	/**
	 * @param {number} index Index
	 * @param {any}    id    ID
	 */
	this.postUpload = ( index: number, id: number ) => {
		this.selectedBatch = {
			batch_id: id,
		};
		this.activeTab = index;
	};

	this.uploadCustomer = () => {
		this.isLoading = true;
		const fd = new FormData();
		fd.append( 'file', this.theFile );
		const req = {
			method: 'POST',
			url: this.batchEndpoint + 'upload',
			data: fd,
			transformRequest: angular.identity,
			headers: {
				// eslint-disable-next-line no-undefined
				'Content-Type': undefined as undefined,
			},
		};
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				this.postUpload( 5, resp.batch_id );
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} ).finally( () => {
				this.isLoading = false;
				Localized.setNotification( {
					type: 'success',
					message: 'File Successfully Uploaded',
				} );
			} );
	};

	/**
	 */
	this.updateBatchProgress = () => {
		const req = {
			method: 'GET',
			url: this.batchEndpoint,
			params: this.sortBatchParams,
		};
		let again = false;
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				this.batches = resp.search_results;
				for ( let i = 0; i < this.batches.length; i++ ) {
					if ( this.batches[ i ].status === 'processing' && this.activeTab === 4 ) {
						again = true;
						break;
					}
				}
				if ( again ) {
					$timeout( () => {
						this.updateBatchProgress();
					}, 5000 );
				}
			}, ( errResp: Error ) => {
				Debug.error( errResp );
			} );
	};

	this.updatePriceSheet = ( priceSheetId: number, isNew: boolean ) => {
		if ( ! isNew ) {
			this.customer.price_sheet_id = priceSheetId;
		} else {
			this.newCustomer.price_sheet_id = priceSheetId;
		}
	};

	this.viewDocument = ( id: number ) => {
		window.open( Admin.Download( Localized.apiURL + '/admin/customers/' + this.customer.id + '/documents/' + id ), '_blank' );
	};

	/**
	 */
	this.removeGroup = () => {
		this.customer.group = null;
		this.customer.group_id = null;
	};

	/**
	 * Handles toggling approver/requestors values that won't mix
	 *
	 * @param {Object}  customer    Customer data
	 * @param {boolean} isRequestor Is the user a requestor
	 */
	this.updateGroupPermissions = ( customer: Customer, isRequestor: boolean ) => {
		if ( customer.group ) {
			if ( isRequestor ) {
				if ( customer.group.requestor ) {
					customer.group.approver = false;
					customer.group.pending_emails = false;
					customer.group.admin = false;
				}
			} else if ( customer.group.approver || customer.group.admin || customer.group.pending_emails ) {
				customer.group.requestor = false;

				if ( customer.group.pending_emails && ! customer.group.approver ) {
					customer.group.pending_emails = false;
				}
			}
		}
	};

	this.OpenSupplierSearch = ( model: any ) => {
		this.searchModalPage = 'supplier';
		let callback = {};
		const data = {
			tabs: [
				{
					http: {
						method: 'GET',
						url: Localized.apiURL + '/admin/punchout/supplier',
						params: {
							q: '',
						},
					},
					relationships: [ 'suppliers' ],
					fields: [ 'id', 'name' ],
					fieldPrefixes: [ 'ID: ', '' ],
					id: 'id',
					selectOne: false,
				},
			],
		};
		callback = {
			confirm( resp: any ) {
				model.punchout_suppliers.push( {
					supplier: {
						name: resp[ 0 ].value.name,
						id: resp[ 0 ].value.id,
					},
				},
				);
			},
			cancel: () => {
				angular.noop();
			},
		};
		SearchModal.Show( callback, 'Suppliers', data, 'Add Supplier' );
	};

	this.removeProfile = ( index: number, customer: any ) => {
		customer.punchout_suppliers[ index ].deleted = true;
	};
}

