import { State, usStates } from '../../../common/usStates';
import template from './customer-address-modal.html';

export const CustomerAddressModalComponent: ng.IComponentOptions = {
	bindings: {
		resolve: '<',
		close: '&',
		dismiss: '&',
	},
	template,
	controller: class CustomerAddressModalController {
		static $inject = [
			'Debug',
			'Localized',
			'Utils',
		]
		key: any;
		resolve: {
			key: any;
			customerId: number;
		};
		customerId: number;
		usStates: State[];
		address: any;
		isAdding: boolean;
		isEditing: boolean;
		isConfirmingDeletion: boolean[];
		isCreating: boolean;
		isDeleting: boolean;
		isLoading: boolean;
		addresses: unknown[];
		isUpdating: boolean;
		close: any;

		constructor(
			private Debug: any,
			private Localized: any,
			private Utils: any,
		) {
			this.usStates = usStates;
		}

		$onInit() {
			this.key = this.resolve.key;
			this.customerId = this.resolve.customerId;
			this.getData();
		}

		add() {
			this.address = {};
			this.isAdding = true;
			this.isEditing = false;
		}

		cancel() {
			this.isAdding = false;
			this.isEditing = false;
			this.address = {};
		}

		/**
		 *
		 * @param {Array} addresses
		 * @return {Array} Addresses with uppercase state property.
		 */
		convertUsStates( addresses:any[] ) {
			return addresses.map( ( address ) => {
				address.state = address.state.toLocaleUpperCase();
				return address;
			} );
		}

		create() {
			this.isCreating = true;
			const endpoint = `${ this.Localized.apiURL }/admin/customers/${ this.customerId }/addresses/${ this.key }/`;
			const params = {};
			const data = this.address;

			this.Utils.httpPost( endpoint, params, data )
				.then( () => {
					this.isCreating = false;
					this.isAdding = false;
					this.getData();
				} );
		}

		edit( e: Event, address: any ) {
			e.preventDefault();
			this.address = address;
			this.isEditing = true;
			this.isAdding = false;
		}

		trash( e: Event, address: any ) {
			this.isDeleting = true;
			e.preventDefault();
			const id = address[ this.key + '_id' ];
			const endpoint = `${ this.Localized.apiURL }/admin/customers/${ this.customerId }/addresses/${ this.key }/${ id }`;
			this.Utils.httpDelete( endpoint )
				.then( () => {
					this.address = {};
					this.isEditing = false;
					this.isDeleting = false;
					this.getData();
				} );
		}

		getData() {
			this.isLoading = true;
			const endpoint =
				`${ this.Localized.apiURL }/admin/customers/${ this.customerId }/addresses/${ this.key }`;

			this.Utils.httpGet( endpoint )
				.then( ( response: any ) => {
					this.addresses = Object.values( response[ `${ this.key }_addresses` ] );
					this.addresses = this.convertUsStates( this.addresses );
				} )
				.catch( ( error: Error ) => {
					this.Debug.error( error );
				} )
				.finally( () => {
					this.isLoading = false;
				} );
		}

		ok() {
			this.close( { $value: 'close' } );
		}

		update() {
			this.isUpdating = true;
			const id = this.address[ this.key + '_id' ];
			const endpoint = `${ this.Localized.apiURL }/admin/customers/${ this.customerId }/addresses/${ this.key }/${ id }`;
			const params = {};
			const data = this.address;

			this.Utils.httpPut( endpoint, params, data )
				.then( () => {
					this.isUpdating = false;
					this.isEditing = false;
					this.getData();
				} );
		}
	},
};
