/**
 * Catalog Component
 *
 * @namespace Components
 */
( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.component( 'vfCustomers', {
			controller: DashboardController,
			templateUrl: `${ localized.dir.templates }/customers.component.html`,
		} );

	DashboardController.$inject = [
		'$rootScope',
		'Admin',
	];

	/**
	 * @param {Object} $rootScope AngularJS service
	 * @param {Object} Admin      VendorFuel service
	 */
	function DashboardController(
		$rootScope: any,
		Admin: any,
	) {
		const vm = this;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			this.breadcrumbs = [ {
				name: 'Customers',
				state: 'customers.page',
			} ];

			vm.isAuthed = Admin.Authed();
			vm.sections = [
				{
					title: 'Customer Accounts',
					link: 'customers-accounts',
					icon: 'admin-users',
					description: 'Manage customer accounts.',
					sublinks: [
						{ title: 'View All Customers', tab: 0 },
						{ title: 'Add New', tab: 1 },
						{ title: 'Upload', tab: 3 },
						{ title: 'Batches', tab: 4 },
						{ title: 'Settings', tab: 6 },
					],
				},
				{
					title: 'Groups',
					link: 'customers-groups',
					icon: 'groups',
					description: 'Manage customer groups.',
					sublinks: [
						{ title: 'View All Groups', tab: 0 },
						{ title: 'Add New', tab: 1 },
						{ title: 'Upload', tab: 3 },
						{ title: 'Batches', tab: 4 },
					],
				},
				{
					title: 'Roles',
					link: 'customers.roles.index',
					icon: 'admin-generic',
					description: 'Manage customer roles.',
					sublinks: [
						{ title: 'View All Roles', state: 'customers.roles.index' },
						{ title: 'Add New', state: 'customers.roles.create' },
					],
				},
			];
		};
	}
}() );
