import React, { useState } from 'react';
import { Button, Card, CardHeader, CardBody, CardFooter, TextControl } from '@wordpress/components';
import { sections } from './sections';

export const DashboardSections = () => {
	const [ filter, setFilter ] = useState<string>( '' );

	const getFilteredSections = ( section: { title: string, description: string } ) => {
		return section.title.toLocaleLowerCase().includes( filter ) || section.description.toLocaleLowerCase().includes( filter );
	};

	const handleClick = () => {
		setFilter( '' );
	};

	return (
		<>
			<div className="row align-items-baseline">
				<div className="col-md-3">
					<h3 className="h5">Sections</h3>
				</div>
				<div className="col-md-9">
					<TextControl type="search" label="Filter sections" hideLabelFromVision value={ filter } onChange={ setFilter } placeholder="Filter sections by title or description." />
				</div>
			</div>
			{ ! sections.filter( getFilteredSections ).length &&
			<>
				<p>No sections found. <Button variant="link" onClick={ handleClick } >Clear filter</Button></p>
			</>
			}
			<div className="row row-cols-1 row-cols-md-3 g-3">
				{ sections.filter( getFilteredSections )
					.map( ( section, index ) =>
						<div key={ index } className="col">
							<div className="card h-100">
								<CardHeader>
									<a className="text-decoration-none" href={ section.href }>
										<i className={ `me-1 bi bi-${ section.icon }` }></i>
										{ section.title }
									</a>
								</CardHeader>
								<CardBody className="h-100">
									{ section.description }
								</CardBody>
								<CardFooter>
									<Button variant="secondary" href={ section.href } >
										Go to { section.title }
									</Button>
								</CardFooter>
							</div>
						</div>,
					)
				}
			</div>
		</>
	);
};
