/**
 * Header Component
 *
 * @namespace Components
 */
( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.component( 'vfHeader', {
			controller: HeaderController,
			templateUrl: `${ localized.dir.templates }/header.component.html`,
		} );

	HeaderController.$inject = [
		'$state',
		'$window',
		'Admin',
		'Localized',
	];

	/**
	 * @param {Object} $state    UI Router object
	 * @param {Object} $window   AngularJS service
	 * @param {Object} Admin     VendorFuel service
	 * @param {Object} Localized VendorFuel service
	 */
	function HeaderController(
		$state: ng.ui.IStateService,
		$window: ng.IWindowService,
		Admin: any,
		Localized: any,
	) {
		const vm = this;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			vm.iconSrc = `${ Localized.root.url }assets/img/vf-icon.svg`;
			vm.iconStyle = {
				height: '2rem',
				verticalAlign: 'baseline',
			};
			vm.pluginVersion = Localized.plugin_data.Version;
			startTutorial();
		};

		/**
		 * Show tutorial based on cookies.
		 */
		function startTutorial() {
			const isShowingTutorial = $window.localStorage.getItem( 'showTutorial' );
			if ( isShowingTutorial === 'true' && Admin.Authed() ) {
				$state.go( 'tutorial' );
			}
		}
	}
}() );
