( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.component( 'manufacturersDetail', {
			controller: ManufacturerController,
			templateUrl: `${ localized.dir.templates }/manufacturers-detail.component.html`,
		} );

	ManufacturerController.$inject = [
		'$http',
		'$rootScope',
		'$scope',
		'$state',
		'$stateParams',
		'Localized',
		'Manufacturers',
	];

	function ManufacturerController(
		$http: ng.IHttpService,
		$rootScope: any,
		$scope: any,
		$state: ng.ui.IStateService,
		$stateParams: ng.ui.IStateParamsService,
		Localized: any,
		Manufacturers: any,
	) {
		const vm = this;
		vm.changeLogo = changeLogo;
		vm.remove = remove;
		vm.removeLogo = removeLogo;
		vm.selectLogo = selectLogo;
		vm.submit = submit;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Manufacturers', state: 'manufacturers.index' },
			];

			if ( $stateParams.id ) {
				this.getManufacturer( $stateParams.id );
			} else {
				this.createManufacturer();
			}
		};

		function changeLogo( el: any ) {
			$scope.$apply( () => {
				vm.manufacturer.logo = el.files[ 0 ];
			} );
		}

		/**
		 */
		this.createManufacturer = () => {
			vm.isNew = true;
			vm.manufacturer = {};
			this.updateBreadcrumb();
		};

		this.getManufacturer = ( id: number ) => {
			vm.isLoading = true;
			Manufacturers.get( { id } ).$promise
				.then( ( manufacturer: any ) => {
					vm.manufacturer = manufacturer;
					if ( vm.manufacturer.images && vm.manufacturer.images.logo ) {
						vm.logoFilename = new URL( vm.manufacturer.images.logo.default ).pathname.split( '/' ).pop();
					}
					this.updateBreadcrumb( vm.manufacturer );
					vm.isLoading = false;
				} );
		};

		/**
		 */
		function remove() {
			vm.isDeleting = true;
			Manufacturers.delete( { id: vm.manufacturer.id } ).$promise
				.then( () => {
					vm.isDeleting = false;
					$state.go( 'manufacturers.index' );
				} );
		}

		function removeLogo( mediaId: number ) {
			vm.isRemovingLogo = true;
			const endpoint = `${ Localized.apiURL }/admin/manufacturers/${ vm.manufacturer.id }/media/${ mediaId }`;
			$http.delete( endpoint )
				.then( () => {
					vm.isRemovingLogo = false;
					$state.reload();
				} );
		}

		function save() {
			vm.isSaving = true;
			Manufacturers.save( vm.manufacturer ).$promise
				.then( ( response: any ) => {
					const id = response.manufacturer.id;
					vm.isSaving = false;
					$state.go( 'manufacturers.show', { id } );
				} );
		}

		/**
		 * Select a logo using WP Media.
		 */
		function selectLogo() {
			// eslint-disable-next-line no-undef
			const fileFrame = wp.media( {
				title: 'Select or Upload image',
				library: {
					type: 'image',
				},
				button: {
					text: 'Select',
				},
				multiple: false,
			} );

			fileFrame.on( 'select', function() {
				$scope.$apply( function() {
					vm.manufacturer.image_url = fileFrame.state().get( 'selection' ).first().toJSON().url;
				} );
			} );

			fileFrame.open();
		}

		/**
		 */
		function submit() {
			if ( vm.isNew ) {
				save();
			} else {
				update();
			}
		}

		/**
		 */
		function update() {
			vm.isUpdating = true;
			Manufacturers.update( { id: vm.manufacturer.id }, vm.manufacturer ).$promise
				.then( () => {
					vm.isUpdating = false;
					$state.reload();
				} );
		}

		/**
		 * @param {Object} manufacturer Manufacturer
		 */
		this.updateBreadcrumb = ( manufacturer?: any ) => {
			if ( manufacturer ) {
				this.breadcrumbs.push( {
					name: manufacturer.name,
					state: 'manufacturers.show({ id: vm.manufacturer.id })',
				} );
			} else {
				this.breadcrumbs.push( {
					name: 'Add New',
					state: 'manufacturers.create',
				} );
			}
		};
	}
}() );
