( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.component( 'manufacturersIndex', {
			controller: ManufacturersController,
			templateUrl: `${ localized.dir.templates }/manufacturers-index.component.html`,
		} );

	ManufacturersController.$inject = [
		'$location',
		'$state',
		'Manufacturers',
	];

	function ManufacturersController(
		$location: ng.ILocationService,
		$state: ng.ui.IStateService,
		Manufacturers: any,
	) {
		const vm = this;
		vm.cancelDelete = cancelDelete;
		vm.changePage = changePage;
		vm.changeQuery = changeQuery;
		vm.changeSortBy = changeSortBy;
		vm.remove = remove;
		vm.submit = submit;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Manufacturers', state: 'manufacturers.index' },
			];
			vm.isConfirmingDeletion = [];
			vm.isDeleting = [];
			vm.params = {
				page: $location.search().page || 1,
				q: $location.search().q || '',
				rpp: 15,
				sortBy: $location.search().sortBy || '',
				sortType: $location.search().sortType || '',
			};
			vm.cachedQ = vm.params.q;
			queryManufacturers();
		};

		function cancelDelete( e: MouseEvent, i: number ) {
			e.preventDefault();
			vm.isConfirmingDeletion[ i ] = false;
		}

		/**
		 * @param {Object} e    Click event
		 * @param {number} page Page number
		 */
		function changePage( e: Event, page: number ) {
			if ( e ) {
				e.preventDefault();
			}
			vm.params.page = page;
			submit();
		}

		/**
		 * @param {string} q Query
		 */
		function changeQuery( q: string ) {
			if ( ! q ) {
				submit();
			}
		}

		/**
		 * @param {string} sortBy Sortby term
		 * @param {Object} e      Click event
		 */
		function changeSortBy( sortBy: string, e: MouseEvent ) {
			e.preventDefault();
			if ( vm.params.sortBy === sortBy ) {
				vm.params.sortType = vm.params.sortType === 'desc' ? 'asc' : 'desc';
			} else {
				vm.params.sortType = 'asc';
				vm.params.sortBy = sortBy;
			}
			updateSearchParams();
			queryManufacturers();
		}

		this.confirmDelete = ( e: MouseEvent, i: number ) => {
			e.preventDefault();
			vm.isConfirmingDeletion[ i ] = true;
		};

		/**
		 * Get the manufacturers
		 */
		function queryManufacturers() {
			vm.isLoading = true;
			Manufacturers.query( vm.params ).$promise
				.then( ( manufacturers: any ) => {
					vm.manufacturers = manufacturers;
					vm.isLoading = false;
				} );
		}

		function remove( e: MouseEvent, id: number, index: number ) {
			e.preventDefault();
			vm.isDeleting[ index ] = true;
			Manufacturers.delete( { id } ).$promise
				.then( () => {
					vm.isDeleting[ index ] = false;
					$state.reload();
				} );
		}

		/**
		 * Submit the search form.
		 */
		function submit() {
			vm.cachedQ = vm.params.q;
			updateSearchParams();
			queryManufacturers();
		}

		/**
		 * Update the search params in the URL.
		 */
		function updateSearchParams() {
			const { params: { q, page } } = vm;
			$location.search( 'q', q ? q : null );
			$location.search( 'page', page > 1 ? page : null );
		}
	}
}() );
