declare let wp: any;

interface Field {
	name: string;
	key: string;
	value: string | number;
}

interface Review {
	id: number;
	'parent_review_id': null | number;
	'customer_id': number;
	'product_id': number;
	status: 'approved' | 'pending';
	rating: number;
	helpfulness: number;
	'display_name': string;
	email: string;
	title: string;
	content: string;
	'created_at': string;
	'updated_at': string;
	'deleted_at': null | string;
}

( function() {
	'use strict';

	angular.module( 'vendorfuelApp' ).component( 'reviewsDetail', {
		controller: ReviewController,
		templateUrl: `${ localized.dir.templates }/reviews-detail.component.html`,
	} );

	ReviewController.$inject = [
		'$rootScope',
		'$state',
		'$stateParams',
		'Reviews',
	];

	function ReviewController(
		$rootScope: any,
		$state: ng.ui.IStateService,
		$stateParams: ng.ui.IStateParamsService,
		Reviews: any,
	) {
		const vm = this;
		vm.approve = approve;
		vm.remove = remove;

		/**
		 * Initialization
		 */
		this.$onInit = () => {
			vm.productSlug = localized.settings.general.product_slug || 'products';
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Products', state: 'catalog-products({ activeTab: 0 })' },
				{ name: 'Reviews', state: 'reviews.index' },
			];

			if ( $stateParams.id ) {
				this.getReview( $stateParams.id );
			}
		};

		function approve() {
			const body = {
				status: 'approved',
			};
			Reviews.update( { id: vm.review.id }, body ).$promise
				.then( () => {
					$state.reload();
				} );
		}

		this.getReview = ( id: number ) => {
			vm.isLoading = true;
			Reviews.get( { id } ).$promise
				.then( ( review: any ) => {
					vm.review = review;
					this.updateBreadcrumb( vm.review );
					vm.isLoading = false;
				} );
		};

		function remove() {
			vm.isDeleting = true;
			Reviews.delete( { id: vm.review.id } ).$promise.then( () => {
				vm.isDeleting = false;
				$state.go( 'reviews.index' );
			} );
		}

		this.updateBreadcrumb = ( review?: Review ) => {
			if ( review ) {
				this.breadcrumbs.push( {
					name: review.title,
					state: 'review.show({ id: vm.review.id })',
				} );
			}
		};
	}
}() );
