import template from './settings-analytics.component.html';

export const SettingsAnalyticsComponent: ng.IComponentOptions = {
	template,
	controller: AnalyticsController,
};

function AnalyticsController(
	Localized,
	Utils,
) {
	const vm = this;
	const wpEndpoint = `${ Localized.wpRestUrl }/settings/analytics`;
	vm.update = update;

	/**
	 * Initialization
	 */
	this.$onInit = () => {
		getData();
	};

	/**
	 */
	function getData() {
		vm.isLoading = true;

		Utils.httpGet( wpEndpoint )
			.then( ( data ) => {
				vm.settings = {
					AW: data.AW,
					conversions: data.conversions,
					UA: data.UA,
					verification: data.verification,
				};
				vm.isLoading = false;
			} );
	}

	/**
	 * @param {Object} settings Options data
	 */
	function update( settings ) {
		vm.isUpdating = true;

		const params = {
			settings: JSON.stringify( settings ),
		};

		Utils.httpPost( wpEndpoint, params )
			.then( ( ) => {
				Localized.setNotification( {
					type: 'success',
					message: 'Analytics settings have been updated.',
				} );
				vm.isUpdating = false;
				getData();
			} );
	}
}

AnalyticsController.$inject = [
	'Localized',
	'Utils',
];
