import template from './settings-options.component.html';

export const SettingsOptionsComponent: ng.IComponentOptions = {
	template,
	controller: OptionsController,
};

function OptionsController(
	Localized,
	Utils,
) {
	const vm = this;
	const apiEndpoint = `${ Localized.apiURL }/stores`;
	const wpEndpoint = `${ Localized.wpRestUrl }/settings/store`;
	vm.update = update;

	/**
	 * Initialization
	 */
	this.$onInit = () => {
		getData();
	};

	/**
	 */
	function getData() {
		vm.isLoading = true;

		Utils.httpGet( apiEndpoint )
			.then( ( response ) => {
				vm.options = processOptions( response.store.options );
				vm.store = response.store;
				vm.isLoading = false;
			} );
	}

	/**
	 * @param {Object} options Settings options
	 * @return {Array} Options
	 */
	function processOptions( options: { [key: string]: boolean; } ) {
		const deprecated = [ 'Load Footer Template', 'Load Menu Nav Template' ];
		return Object.entries( options )
			.filter( ( option ) => {
				const [ key ] = option;
				return ! deprecated.includes( key );
			} )
			.map( ( option ) => {
				const [ key, value ] = option;
				return {
					key, value,
				};
			} );
	}

	/**
	 * @param {Object} data Options data
	 */
	function update( data ) {
		vm.isUpdating = true;
		const options = Object.fromEntries( data.map( ( option ) => {
			return [ option.key, option.value ];
		} ) );

		const body = {
			options,
		};

		Utils.httpPut( apiEndpoint, null, body )
			.then( ( ) => {
				const params = body;
				Utils.httpPost( wpEndpoint, params )
					.then( ( ) => {
						Localized.setNotification( {
							type: 'success',
							message: 'Setting options have been updated.',
						} );
						vm.isUpdating = false;
						getData();
					} );
			} );
	}
}

OptionsController.$inject = [
	'Localized',
	'Utils',
];
