import template from './settings-plugin.component.html';

export const SettingsPluginComponent: ng.IComponentOptions = {
	controller: PluginController,
	template,
};

interface Settings {
	'api_url': string,
	'api_key': string,
	bootstrap: boolean,
	'product_slug': string,
	'cat_slug': string,
	requireAddress: boolean,
	debug: boolean,
	redirectToCart: boolean,
	pagination: boolean,
	enableRecyclable: boolean,
	checkout: {
		'attention_option': boolean;
		'company_name_option': boolean;
		'cost_center_option': boolean;
		'issuing_office_option': boolean;
		'notes_option': boolean;
		'purchase_order_option': boolean;
	},
	showSubcategoryCards: boolean
  }

function PluginController(
	Localized,
	Utils,
) {
	const vm = this;
	const wpEndpoint = `${ Localized.wpRestUrl }/settings/general`;
	vm.update = update;

	this.$onInit = () => {
		getData();
	};

	/**
	 */
	function getData() {
		vm.isLoading = true;

		Utils.httpGet( wpEndpoint )
			.then( ( settings: Settings ) => {
				vm.settings = settings;
				vm.isLoading = false;
			} );
	}

	/**
	 * @param {Object} settings Options data
	 */
	function update( settings: Settings ) {
		vm.isUpdating = true;

		const params = {
			settings: JSON.stringify( settings ),
		};

		Utils.httpPost( wpEndpoint, params )
			.then( ( ) => {
				Localized.setNotification( {
					type: 'success',
					message: 'Plugin settings have been updated.',
				} );
				vm.isUpdating = false;
				getData();
			} );
	}
}

PluginController.$inject = [
	'Localized',
	'Utils',
];
