import { usStates } from '../../../common/usStates';
import template from './settings-store.component.html';

export const SettingsStoreComponent: ng.IComponentOptions = {
	bindings: {
		getMetrc: '&',
	},
	controller: StoreController,
	template,
};

function StoreController(
	Localized,
	Utils,
) {
	const vm = this;
	const apiEndpoint = `${ Localized.apiURL }/stores`;
	const wpEndpoint = `${ Localized.wpRestUrl }/settings/store`;
	vm.update = update;

	/**
	 * Initialization
	 */
	this.$onInit = () => {
		vm.stateOptions = usStates;
		getData();
	};

	/**
	 */
	function getData() {
		vm.isLoading = true;

		Utils.httpGet( apiEndpoint )
			.then( ( response ) => {
				vm.store = response.store;
				vm.getMetrc( { hasMetrc: response.store.metrc_enabled } );
				vm.isLoading = false;
			} );
	}

	/**
	 * @param {Object} store Store data
	 */
	function update( store ) {
		vm.isUpdating = true;
		const body = new FormData();
		body.append( 'name', store.name );
		body.append( 'url', store.url );

		const request = {
			method: 'POST',
			url: apiEndpoint,
			data: body,
			transformRequest: angular.identity,
			headers: {
				// eslint-disable-next-line no-undefined
				'Content-Type': undefined,
			},
		};

		Utils.getHttpPromise( request )
			.then( ( ) => {
				// After updating the VF API, update the WP API with minimal data that's currently used.
				// eslint-disable-next-line no-shadow
				const body = {
					name: store.name,
					url: store.url,
					metrc_enabled: store.metrc_enabled,
				};

				Utils.httpPost( wpEndpoint, null, body )
					.then( ( ) => {
						Localized.setNotification( {
							type: 'success',
							message: 'Store settings have been updated.',
						} );
						vm.isUpdating = false;
						getData();
					} );
			} );
	}
}

StoreController.$inject = [
	'Localized',
	'Utils',
];
