import React, { SyntheticEvent } from 'react';
import { Button, FormTokenField } from '@wordpress/components';
import { stateOptions } from './stateOptions';
import { vfAPI } from '../../admin/core/vfAPI';

// eslint-disable-next-line @typescript-eslint/no-empty-interface
interface Props {}

interface State {
	isBusy: boolean;
	taxStates: string[];
	taxStateNames: string[];
}

const stateNames = stateOptions.map( ( state ) => state.name );
const stateValues = stateOptions.map( ( state ) => state.value );
const url = `${ localized.apiURL }/admin/tax-states`;

export class TaxesOptions extends React.Component<Props, State> {
	constructor( props: Props ) {
		super( props );

		this.state = {
			isBusy: false,
			taxStates: [],
			taxStateNames: [],
		};

		this.addAllStates = this.addAllStates.bind( this );
		this.handleSubmit = this.handleSubmit.bind( this );
		this.removeAllStates = this.removeAllStates.bind( this );
		this.setSelectedStates = this.setSelectedStates.bind( this );
	}

	componentDidMount() {
		this.getData();
	}

	addAllStates() {
		this.setState( { taxStates: stateValues } );
		this.setState( { taxStateNames: stateNames } );
	}

	getData() {
		this.setState( { isBusy: true } );
		vfAPI.get( url )
			.then( ( response ) => {
				if ( response.data.tax_states ) {
					this.setState( { taxStates: response.data.tax_states } );
					this.setState( { taxStateNames: this.getStateNames( response.data.tax_states ) } );
					this.setState( { isBusy: false } );
				}
			} );
	}

	getStateNames( values: string[] ): string[] {
		return values.map( ( value ) => {
			return stateOptions.find( ( state ) => state.value === value ).name;
		} );
	}

	getStateValues( tokens: string[] ): string[] {
		return tokens.map( ( token ) => {
			return stateOptions.find( ( state ) => state.name === token ).value;
		} );
	}

	handleSubmit( e: SyntheticEvent ) {
		e.preventDefault();
		this.setState( { isBusy: true } );
		const data = {
			state_id: this.state.taxStates,
		};
		vfAPI.post( url, data )
			.then( ( response ) => {
				if ( response.data.success ) {
					this.setState( { isBusy: false } );
				}
			} );
	}

	removeAllStates() {
		this.setState( { taxStates: [] } );
		this.setState( { taxStateNames: [] } );
	}

	setSelectedStates( tokens: string[] ) {
		const allowedTokens = tokens.filter( ( token ) => stateNames.includes( token ) );
		this.setState( { taxStateNames: allowedTokens } );
		this.setState( { taxStates: this.getStateValues( allowedTokens ) } );
	}

	render() {
		return (
			<form onSubmit={ this.handleSubmit }>
				<fieldset disabled={ this.state.isBusy }>
					<legend>States to collect taxes from</legend>
					<FormTokenField label="Add States" value={ this.state.taxStateNames } suggestions={ stateNames } onChange={ ( tokens ) => this.setSelectedStates( tokens ) } placeholder="California, Oklahoma, District of Columbia, etc." />
					<div className="btn-toolbar justify-content-between gap-1">
						<Button variant="primary" type="submit" isBusy={ this.state.isBusy }>Update</Button>
						<div className="hstack gap-1">
							<Button variant="secondary" onClick={ this.addAllStates } isBusy={ this.state.isBusy }>Add All States</Button>
							<Button variant="tertiary" isDestructive onClick={ this.removeAllStates } isBusy={ this.state.isBusy }>Remove All States</Button>
						</div>
					</div>
				</fieldset>
			</form>
		);
	}
}
