import template from './taxes-avalara.component.html';

export const TaxesAvalaraComponent: ng.IComponentOptions = {
	template,
	controller: class TaxesAvalaraController {
		static $inject: string[] = [
			'$window',
			'AlertModal',
			'Localized',
			'Utils',
		];
		baseURL: string;
		isDeleting: boolean;
		isLoading: boolean;
		form: any;
		isGettingLog: boolean;
		isUpdating: boolean;

		constructor(
			private $window: ng.IWindowService,
			private AlertModal: any,
			private Localized: any,
			private Utils: any,
		) {
			this.baseURL = `${ localized.apiURL }/admin/tax/avalara`;
		}

		$onInit() {
			this.getData();
		}

		deleteData() {
			this.isDeleting = true;
			this.Utils.httpDelete( this.baseURL )
				.then( () => {
					this.form = {};
					this.isDeleting = false;
				} );
		}

		getData() {
			this.isLoading = true;
			this.Utils.httpGet( `${ this.baseURL }/config` )
				.then( ( response: any ) => {
					this.form = response.avalara_config;
					this.isLoading = false;
				} );
		}

		test() {
			this.isLoading = true;
			this.Utils.httpPost( `${ this.baseURL }/test`, this.form )
				.then( ( response: any ) => {
					let testMessage = 'Avalara connection test successful!';
					if ( response.errors.length ) {
						testMessage = 'Avalara connection test failed!';
					}
					const callback = {
						confirm() {
							this.isLoading = false;
						},
					};
					this.AlertModal.Show( callback, 'Connection Test Results', testMessage, 'Ok' );
				} );
		}

		update() {
			this.isUpdating = true;
			const data = { ...this.form };

			this.Utils.httpPut( `${ this.baseURL }/config`, null, data )
				.then( ( response: any ) => {
					if ( response.errors.length ) {
						response.errors.forEach( ( message: string ) => {
							this.Localized.setNotification( {
								type: 'error',
								message,
							} );
						} );
					} else {
						this.Localized.setNotification( {
							type: 'success',
							message: 'Avalara settings updated.',
						} );
					}
				} )
				.finally( () => {
					this.isUpdating = false;
				} );
		}

		viewLog() {
			this.isGettingLog = true;
			this.Utils.httpPost( `${ this.baseURL }/log`, this.form )
				.then( ( response: any ) => {
					if ( response.log ) {
						const callback = {
							confirm() {
								angular.noop();
							},
						};
						this.AlertModal.Show( callback, 'Connection Test Results', response.log, 'Ok' );
					} else {
						this.Localized.setNotification( {
							type: 'info',
							message: 'No saved logs found.',
						} );
						this.$window.scrollTo( {
							top: 0,
							left: 0,
							behavior: 'smooth',
						} );
					}
				} )
				.finally( () => {
					this.isGettingLog = false;
				} );
		}
	},
};
