import { tinymceOptions } from '../common/tinymce-options';

( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'CatalogBannersController', CatalogBannersController );

	CatalogBannersController.$inject = [
		'$scope',
		'$rootScope',
		'$stateParams',
		'Admin',
		'ConfirmModal',
		'Settings',
		'Debug',
		'Utils',
		'Localized',
	];

	/**
	 * @param {Object} $scope       Angular service
	 * @param {Object} $rootScope   Angular service
	 * @param {Object} $stateParams UI Router service
	 * @param {Object} Admin        VendorFuel service
	 * @param {Object} ConfirmModal VendorFuel service
	 * @param {Object} Settings     VendorFuel service
	 * @param {Object} Debug        VendorFuel service
	 * @param {Object} Utils        VendorFuel service
	 * @param {Object} Localized    VendorFuel service
	 */
	function CatalogBannersController(
		$scope: any,
		$rootScope: any,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		ConfirmModal: any,
		Settings: any,
		Debug: any,
		Utils: any,
		Localized: any,
	) {
		this.tinymceOptions = tinymceOptions;

		/**
		 * Initialization
		 */
		this.init = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Banners', state: 'catalog-banners' },
			];

			$scope.activeTab = parseInt( $stateParams.activeTab ) || 0;
			$scope.bannerAreaEndpoint = Localized.apiURL + '/admin/banner-areas/';
			$scope.bannerEndpoint = Localized.apiURL + '/admin/banner/';
			$scope.isAuthed = Admin.Authed();
			$scope.loading = false;
			$scope.loadingMore = false;
			$scope.newBanner = {};
			$scope.newBannerArea = {};
			$scope.object = Object;
			$scope.rppValues = [ 15, 30, 50, 100 ];
			$scope.per_page = $scope.rppValues[ 0 ]; // Must come after $scope.rppValues declaration.
			$scope.searchParams = {
				q: '',
				sortBy: '',
				sortType: '',
				rpp: $scope.rppValues[ 0 ],
			};
			$scope.searchTerm = '';
			$scope.selectedBanner = null;
			$scope.selectedBannerArea = null;
			$scope.sortAscending = true;
			$scope.TinyMCEOptions = {
				theme: 'modern',
				// skin: 'lightgray',
				plugins: 'advlist autolink link image lists charmap print preview fullscreen code',
				toolbar: 'numlist bullist | undo redo | styleselect | bold italic | link image alignleft aligncenter alignright | preview code | fullscreen',
			};
		};
		this.init();

		//FUNCTIONS
		$scope.LoginCallback = function() {
			$scope.loading = true;
			//logged in, do something. i.e. make api calls to load current tab's data
		};
		$scope.TabChanged = function( i: number ) {
			$scope.activeTab = i;
			Settings.errors = {};
			$scope.loading = true;
			$scope.saving = false;
			$scope.saved = false;
			$scope.cancelled = false;
			const req = {
				method: 'GET',
				url: $scope.bannerAreaEndpoint,
			};
			switch ( i ) {
				case 0:
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.bannerAreas = resp.banner_areas;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
				case 1:
					$scope.loading = false;
					break;
				case 2:

					if ( $scope.selectedBannerArea ) {
						$scope.newBanner.area_id = $scope.selectedBannerArea.area_id;
					}
					$scope.loading = false;
					break;
				case 3:
					req.url += $scope.selectedBannerArea.area_id;
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.selectedBannerArea = resp.banner_area;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
				case 4:

					$scope.loading = false;
					break;
			}
		};
		$scope.AddBanner = function() {
			$scope.addingBanner = true;
			const req = {
				method: 'POST',
				url: $scope.bannerEndpoint,
				data: $scope.newBanner,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				if ( resp.errors.length <= 0 ) {
					$scope.activeTab = 3;
					$scope.newBanner = {};
				}
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.addingBanner = false;
			} );
		};
		$scope.AddBannerArea = function() {
			$scope.loading = true;
			const req = {
				method: 'POST',
				url: $scope.bannerAreaEndpoint,
				data: $scope.newBannerArea,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				if ( ! resp.errors.length ) {
					$scope.selectedBannerArea = {
						area_id: resp.area_id,
					};
					$scope.activeTab = 3;
					$scope.newBannerArea = {};
				}
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};
		$scope.ChangeTab = function( tabIndex: number, index: number ) {
			$scope.activeTab = tabIndex;
			$scope.selectedBannerArea = $scope.bannerAreas.data[ index ];
		};
		$scope.ChangeBannerTab = function( tabIndex: number, index: number ) {
			$scope.activeTab = tabIndex;
			$scope.selectedBanner = $scope.selectedBannerArea.banners[ index ];
		};
		$scope.RemoveBanner = function() {
			const callback = {
				confirm() {
					$scope.loading = true;
					const params = {
						bid: $scope.selectedBanner.banner_id,
					};
					const req = {
						method: 'DELETE',
						url: $scope.bannerEndpoint,
						data: params,
					};
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						if ( resp.errors.length <= 0 ) {
							$scope.selectedBanner = {};
							$scope.activeTab = 3;
						}
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
				},
				cancel() {

				},
			};
			ConfirmModal.Show( callback, 'Delete Banner?', 'This will completely remove the banner from the store. This action cannot be undone.', 'Back', 'DELETE' );
		};

		$scope.RemoveBannerArea = function() {
			const callback = {
				confirm() {
					$scope.loading = true;
					const req = {
						method: 'DELETE',
						url: $scope.bannerAreaEndpoint + $scope.selectedBannerArea.area_id,
					};
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						if ( ! resp.errors.length ) {
							$scope.selectedBannerArea = null;
							$scope.activeTab = 0;
						}
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
				},
				cancel() {

				},
			};
			ConfirmModal.Show( callback, 'Delete Banner Area?', 'This will delete the area and all banners associated with it. This cannot be undone.', 'Back', 'DELETE' );
		};
		$scope.SearchResults = function( resp: any ) { //callback that is used by the search modal
			if ( $scope.activeTab === 2 ) {
				$scope.addParams.products = [];
				resp.forEach( function( item: any ) {
					$scope.addParams.products.push( item.value );
				} );
			}
			if ( $scope.activeTab === 3 ) {
				$scope.editedProducts = [];
				resp.forEach( function( item: any ) {
					$scope.editedProducts.push( item.value.product_id );
				} );
			}
		};
		$scope.SortIndex = function( sortBy: string ) {
			$scope.sortAscending = ( $scope.searchParams.sortBy === sortBy ) ? ! $scope.sortAscending : true;
			$scope.searchParams.sortBy = sortBy;
			$scope.searchParams.sortType = $scope.sortAscending ? 'asc' : 'desc';
			$scope.SearchBannerAreas( $scope.searchParams.page );
		};

		$scope.SearchBannerAreas = function( page: number, query: string ) {
			$scope.loadingMore = true;
			$scope.searchParams.q = query;
			$scope.searchTerm = $scope.searchParams.q;
			$scope.searchParams.page = page || 1;
			const req = {
				method: 'GET',
				url: $scope.bannerAreaEndpoint,
				params: $scope.searchParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.bannerAreas = resp.banner_areas;
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.loadingMore = false;
			} );
		};

		$scope.UpdateBannerArea = function() {
			$scope.updatingArea = true;
			const req = {
				method: 'PUT',
				url: $scope.bannerAreaEndpoint + $scope.selectedBannerArea.area_id,
				data: $scope.selectedBannerArea,
			};
			Utils.getHttpPromise( req ).then( function() {

			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.updatingArea = false;
			} );
		};
		$scope.UpdateBannerContent = function() {
			$scope.updatingBanner = true;
			// var params = {};
			const params = $scope.selectedBanner;
			params.bid = $scope.selectedBanner.banner_id;
			const req = {
				method: 'PUT',
				url: $scope.bannerEndpoint + $scope.selectedBanner.banner_id,
				data: params,
			};
			Utils.getHttpPromise( req ).then( function() {

			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.updatingBanner = false;
			} );
		};
	}
}() );

