import { tinymceOptions } from '../common/tinymce-options';

( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'CatalogCollectionsController', CatalogCollectionsController );

	CatalogCollectionsController.$inject = [
		'$scope',
		'$rootScope',
		'$state',
		'$stateParams',
		'Admin',
		'Settings',
		'Debug',
		'Utils',
		'Localized',
		'SearchModal',
		'ViewModal',
	];

	/**
	 * @param {Object} $scope       Angular service
	 * @param {Object} $rootScope   Angular service
	 * @param {Object} $state       UI Router service
	 * @param {Object} $stateParams UI Router service
	 * @param {Object} Admin        VendorFuel service
	 * @param {Object} Settings     VendorFuel service
	 * @param {Object} Debug        VendorFuel service
	 * @param {Object} Utils        VendorFuel service
	 * @param {Object} Localized    VendorFuel service
	 * @param {Object} SearchModal  VendorFuel service
	 * @param {Object} ViewModal    VendorFuel service
	 */
	function CatalogCollectionsController(
		$scope: any,
		$rootScope: any,
		$state: ng.ui.IStateService,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		Settings: any,
		Debug: any,
		Utils: any,
		Localized: any,
		SearchModal: any,
		ViewModal: any,
	) {
		const vm = this;
		this.tinymceOptions = tinymceOptions;

		vm.changeSortBy = changeSortBy;

		/**
		 * Initialization
		 */
		this.init = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Collections', state: 'catalog-collection' },
			];

			$scope.activeTab = parseInt( $stateParams.activeTab );
			$scope.addParams = {
				products: [],
				categories: [],
			};
			$scope.categoryEndpoint = Localized.apiURL + '/admin/category/';
			$scope.collectionEndpoint = Localized.apiURL + '/admin/product/collection/';
			$scope.editedCategories = [];
			$scope.editedProducts = [];
			$scope.editName = false;
			$scope.isAuthed = Admin.Authed();
			$scope.loading = false;
			$scope.loadingMore = false;
			$scope.productEndpoint = Localized.apiURL + '/admin/products/';
			$scope.productsLoading = false;
			$scope.object = Object;
			$scope.rppValues = [ 15, 30, 50, 100 ];
			$scope.per_page = $scope.rppValues[ 0 ]; // Must be defined after rppValues declaration.
			$scope.saved = false;
			$scope.saving = false;
			$scope.searchParams = {};
			$scope.searchTerm = '';
			$scope.searchTypes = {
				PRODUCTS: 'PRODUCTS',
				CATEGORIES: 'CATEGORIES',
			};
			$scope.searchType = $scope.searchTypes.PRODUCTS; // Must be defined after searchTypes declaration.
			$scope.selectedCollection = null; //Collection to show in the view tab
			$scope.showChanges = false;

			vm.allColSearchParams = {
				q: '',
				sortBy: '',
				sortType: '',
				rpp: $scope.rppValues[ 0 ], // Must be defined after rppValues declaration.
			};
			vm.sortAscending = true;

			//If the view tab is selected, redirect to the search page since selected product is null
			if ( $scope.activeTab === 2 ) {
				$scope.activeTab = 0;
				$scope.TransitionState();
			}
		};
		this.init();

		//FUNCTIONS
		$scope.LoginCallback = function() {
			$scope.loading = true;
			//logged in, do something. i.e. make api calls to load current tab's data
		};
		$scope.ChangeEdit = function() {
			$scope.editName = ! $scope.editName;
		};
		$scope.SearchResults = function( resp: any ) { //callback that is used by the search modal
			if ( $scope.activeTab === 1 ) {
				switch ( $scope.searchType ) {
					case $scope.searchTypes.PRODUCTS:
						$scope.addParams.products = resp;
						break;
					case $scope.searchTypes.CATEGORIES:
						$scope.addParams.categories = resp;
						break;
				}
			} else if ( $scope.activeTab === 2 ) {
				switch ( $scope.searchType ) {
					case $scope.searchTypes.PRODUCTS:
						$scope.editedProducts = resp;
						break;
					case $scope.searchTypes.CATEGORIES:
						$scope.editedCategories = resp;
						break;
				}
			}
		};
		$scope.SearchCancelled = function() { //callback that is used by the search modal

		};
		$scope.TransitionState = function() {
			$state.go( 'catalog-collections', { activeTab: $scope.activeTab } );
		};
		$scope.TabChanged = function( i: number ) {
			$scope.activeTab = i;
			Settings.errors = {};
			$scope.loading = true;
			$scope.saving = false;
			$scope.saved = false;
			$scope.cancelled = false;
			const req = {
				method: 'GET',
				url: $scope.collectionEndpoint,
			};
			switch ( i ) {
				case 0:
					//First tab selected
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.collections = resp.collections;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
				case 1:
					//Second tab selected

					$scope.loading = false;
					break;
				case 2:
					req.url += $scope.selectedCollection.id;
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.selectedCollection = resp.collection;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
			}
		};
		$scope.ChangeTab = function( tabIndex: number, collectionIndex: number ) {
			$scope.activeTab = tabIndex;
			$scope.selectedCollection = $scope.collections.data[ collectionIndex ];
		};

		$scope.OpenSearchModalUpdate = function() {
			$scope.searchType = $scope.searchTypes.PRODUCTS;
			let callback = {};
			const data = {
				tabs: [
					{
						http: {
							method: 'GET',
							url: $scope.productEndpoint,
							params: {
								q: '',
								excludedField: 'collection_id',
								excludedId: $scope.selectedCollection.id,
								excludedTable: 'product_collection',
							},
						},
						relationships: [ 'products' ],
						fields: [ 'sku', 'image', 'description', 'status' ],
						fieldPrefixes: [ '', '', '', 'Status: ' ],
						id: 'product_id',
						selectOne: false,
					},
					{
						http: {
							method: 'GET',
							url: $scope.collectionEndpoint + $scope.selectedCollection.id,
							params: {
								q: '',
								'searchFields[]': [ 'description', 'sku' ],
							},
						},
						relationships: [ 'collection', 'products' ],
						fields: [ 'sku', 'image', 'description', 'status' ],
						fieldPrefixes: [ '', '', '', 'Status: ' ],
						id: 'product_id',
					},
				],
				updatedItems: $scope.editedProducts,
			};
			callback = {
				confirm: $scope.SearchResults,
				cancel: $scope.SearchCancelled,
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.selectedCollection.name, data, 'Add Products' );
		};
		$scope.OpenSearchModalAdd = function() {
			$scope.searchType = $scope.searchTypes.PRODUCTS;
			let callback = {};
			const data = {
				tabs: [
					{
						http: {
							method: 'GET',
							url: $scope.productEndpoint,
							params: {
								q: '',
							},
						},
						relationships: [ 'products' ],
						fields: [ 'sku', 'image', 'description', 'status' ],
						fieldPrefixes: [ '', '', '', 'Status: ' ],
						id: 'product_id',
						selectOne: false,
					},
				],
				updatedItems: $scope.addParams.products,
			};
			callback = {
				confirm: $scope.SearchResults,
				cancel: $scope.SearchCancelled,
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.addParams.name || 'New Collection', data, 'Add Products' );
		};
		$scope.OpenSearchModalCategoryUpdate = function() {
			$scope.searchType = $scope.searchTypes.CATEGORIES;
			let callback = {};
			const data = {
				tabs: [
					{
						http: {
							method: 'GET',
							url: $scope.categoryEndpoint,
							params: {
								q: '',
								excludedField: 'collection_id',
								excludedId: $scope.selectedCollection.id,
								excludedTable: 'category_collection',
							},
						},
						relationships: [ 'categories' ],
						fields: [ 'title' ],
						fieldPrefixes: [ '' ],
						id: 'cat_id',
						selectOne: false,
					},
					{
						http: {
							method: 'GET',
							url: $scope.collectionEndpoint + $scope.selectedCollection.id,
							params: {
								q: '',
								'searchFields[]': [ 'description', 'title' ],
							},
						},
						relationships: [ 'collection', 'categories' ],
						fields: [ 'title' ],
						fieldPrefixes: [ '' ],
						id: 'cat_id',
					},
				],
				updatedItems: $scope.editedCategories,
			};
			callback = {
				confirm: $scope.SearchResults,
				cancel: $scope.SearchCancelled,
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.selectedCollection.name, data, 'Add Categories' );
		};
		$scope.OpenSearchModalCategoryAdd = function() {
			$scope.searchType = $scope.searchTypes.CATEGORIES;
			let callback = {};
			const data = {
				tabs: [
					{
						http: {
							method: 'GET',
							url: $scope.categoryEndpoint,
							params: {
								q: '',
							},
						},
						relationships: [ 'categories' ],
						fields: [ 'description', 'title' ],
						fieldPrefixes: [ '', '' ],
						id: 'cat_id',
						selectOne: false,
					},
				],
				updatedItems: $scope.addParams.categories,
			};
			callback = {
				confirm: $scope.SearchResults,
				cancel: $scope.SearchCancelled,
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.addParams.name || 'New Collection', data, 'Add Categories' );
		};
		$scope.OpenViewModal = function( product: any ) {
			$scope.callback = {};
			const callback = {
				confirm() {
					$scope.TabChanged( 2 );
				},
				cancel: $scope.SearchCancelled,
			};
			ViewModal.Show( callback, 'Quick View: ' + product.sku, product );
		};

		/**
		 * @param {string} sortBy Sortby term
		 * @param {Object} e      Click event
		 */
		function changeSortBy( sortBy: string, e: Event ) {
			e.preventDefault();
			vm.sortAscending = ( vm.allColSearchParams.sortBy === sortBy ) ? ! vm.sortAscending : true;
			vm.allColSearchParams.sortBy = sortBy;
			vm.allColSearchParams.sortType = vm.sortAscending ? 'asc' : 'desc';
			$scope.SearchCollections( vm.allColSearchParams.page );
		}

		$scope.SearchCollections = function( page: number, query: string ) {
			$scope.loadingMore = true;
			vm.allColSearchParams.q = query;
			$scope.searchTerm = vm.allColSearchParams.q;
			vm.allColSearchParams.page = page || 1;
			const req = {
				method: 'GET',
				url: $scope.collectionEndpoint,
				params: vm.allColSearchParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.collections = resp.collections;
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.loadingMore = false;
			} );
		};
		$scope.SearchCollection = function( rpp: number, page: number ) {
			$scope.productsLoading = true;
			$scope.searchParams.rpp = rpp;
			$scope.searchParams.page = page || 1;
			$scope.searchParams[ 'searchFields[]' ] = [ 'description', 'sku' ];
			const req = {
				method: 'GET',
				url: $scope.collectionEndpoint + $scope.selectedCollection.id,
				params: $scope.searchParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.selectedCollection = resp.collection;
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.productsLoading = false;
			} );
		};
		$scope.AddCollection = function( id: number | null ) { // If id is null, we are adding new collection
			$scope.loading = true;
			const updateParams: any = {
				products: [],
				categories: [],
			};
			let error = false;
			let req: any = {};
			if ( id ) {
				updateParams.name = $scope.selectedCollection.name;
				updateParams.image = $scope.selectedCollection.img_url;
				updateParams.description = $scope.selectedCollection.description;
				updateParams.products = $scope.editedProducts.map( function( product: any ) {
					return product.value.product_id;
				} );
				updateParams.categories = $scope.editedCategories.map( function( category: any ) {
					return category.value.cat_id;
				} );
				req = {
					method: 'PUT',
					url: $scope.collectionEndpoint + id,
					data: updateParams,
				};
			} else if ( $scope.addParams.name && ( $scope.addParams.products.length || $scope.addParams.categories.length ) ) {
				for ( let i = 0; i < $scope.addParams.products.length; i++ ) {
					updateParams.products.push( $scope.addParams.products[ i ].value.product_id );
				}
				for ( let j = 0; j < $scope.addParams.categories.length; j++ ) {
					updateParams.categories.push( $scope.addParams.categories[ j ].value.cat_id );
				}
				updateParams.name = $scope.addParams.name;
				updateParams.description = $scope.addParams.description;
				updateParams.image = $scope.addParams.img_url;
				req = {
					method: 'POST',
					url: $scope.collectionEndpoint,
					data: updateParams,
				};
			} else {
				error = true;
			}
			if ( ! error ) {
				Utils.getHttpPromise( req ).then( function( resp: any ) {
					if ( ! resp.errors.length ) {
						if ( ! id ) {
							$scope.selectedCollection = {
								id: resp.collection_id,
								name: req.data.name,
								description: req.data.description,
							};
							$scope.addParams = {
								products: [],
								categories: [],
							};
							$scope.activeTab = 2;
						} else {
							$scope.TabChanged( 2 );
							$scope.showChanges = false;
							$scope.editedProducts = [];
							$scope.editedCategories = [];
						}
					}
				}, function( errResp: any ) {
					Debug.error( errResp );
				} ).finally( function() {
					$scope.loading = false;
				} );
			} else {
				$scope.loading = false;
			}
		};
		$scope.DeleteCollection = function( id: number ) {
			const callback = {
				confirm() {
					$scope.loading = true;
					const req = {
						method: 'DELETE',
						url: $scope.collectionEndpoint + id,
					};
					Utils.getHttpPromise( req ).then( function() {
						$scope.activeTab = 0;
						$scope.selectedCollection = null;
					}, function( errResp: any ) {
						Debug.error( errResp );
					} );
				},
				cancel() { },
			};
			$scope.confirm.Show( callback, 'Delete Collection?', 'This will delete your collection and any products associated with it.', 'Back', 'DELETE' );
		};
		$scope.ShowChanges = function() {
			$scope.showChanges = ! $scope.showChanges;
		};
		$scope.ChangeView = function() {
			$scope.viewProducts = ! $scope.viewProducts;
		};
		$scope.OpenMediaFrame = function( image: string ) {
			// eslint-disable-next-line no-undef
			const fileFrame = wp.media( {
				title: 'Select or Upload Logo image',
				library: {
					type: 'image',
				},
				button: {
					text: 'Select',
				},
				multiple: false,
			} );
			fileFrame.on( 'select', function() {
				$scope.$apply( function() {
					$scope.attachment = fileFrame
						.state()
						.get( 'selection' )
						.first()
						.toJSON();
					$scope.removingImage = true;
					if ( image === 'add_img' ) {
						$scope.addParams.img_url = $scope.attachment.url;
					} else if ( image === 'update_img' ) {
						$scope.selectedCollection.img_url = $scope.attachment.url;
					}
				} );
			} );
			fileFrame.open();
		};
	}
}() );

