( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'CatalogPromoCodesController', CatalogPromoCodesController );

	CatalogPromoCodesController.$inject = [
		'$scope',
		'$rootScope',
		'$stateParams',
		'Admin',
		'Settings',
		'Debug',
		'Utils',
		'Localized',
		'SearchModal',
		'PromoCodesService',
	];

	/**
	 * @param {Object} $scope            Angular service
	 * @param {Object} $rootScope        Angular service
	 * @param {Object} $stateParams      UI Router service
	 * @param {Object} Admin             VendorFuel service
	 * @param {Object} Settings          VendorFuel service
	 * @param {Object} Debug             VendorFuel service
	 * @param {Object} Utils             VendorFuel service
	 * @param {Object} Localized         VendorFuel service
	 * @param {Object} SearchModal       VendorFuel service
	 * @param {Object} PromoCodesService service
	 */
	function CatalogPromoCodesController(
		$scope: any,
		$rootScope: any,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		Settings: any,
		Debug: any,
		Utils: any,
		Localized: any,
		SearchModal: any,
		PromoCodesService: any,
	) {
		const vm = this;

		this.init = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'Promo Codes', state: 'catalog-promocodes' },
			];

			$scope.activeTab = parseInt( $stateParams.activeTab );
			$scope.addParams = {
				auto_add: false,
				require_all_items: false,
				products: {
					data: [],
				},
			};
			$scope.customerEndpoint = Localized.apiURL + '/admin/customers/';
			$scope.discountTypes = {
				percentage: 'Percentage',
				discount: 'Discount',
				// 'price': 'Price'
			};
			$scope.addParams.discount_type = $scope.discountTypes.percentage; // Must be defined after discountTypes declared;
			$scope.isAuthed = Admin.Authed();
			$scope.loading = false;
			$scope.loadingMore = false;
			$scope.object = Object;
			$scope.productEndpoint = Localized.apiURL + '/admin/products/';
			$scope.promocodeEndpoint = Localized.apiURL + '/admin/promo-codes/';
			$scope.rppValues = [ 15, 30, 50, 100 ];
			$scope.per_page = $scope.rppValues[ 0 ]; // Must be defined after rppValues is declared.
			$scope.promoCode = {};
			$scope.promoCodes = [];
			$scope.saved = false;
			$scope.saving = false;
			$scope.searchProductParams = {
				q: '',
				rpp: $scope.rppValues[ 0 ], // Must be defined after rppValues is declared.
				searchFields: [ 'sku', 'description', 'product_id' ],
			};
			$scope.searchTerm = '';
			$scope.selectedPromocode = null;
			$scope.showChanges = false;
			$scope.updatedProducts = [];
			$scope.addedProducts = [];

			vm.searchParams = {
				page: 1,
				q: '',
				sortBy: '',
				sortType: '',
				rpp: $scope.rppValues[ 0 ], // Must be defined after rppValues is declared.
			};
			vm.sortAscending = true;

			//If the view tab is selected, redirect to the search page since selected product is null
			if ( $scope.activeTab === 2 ) {
				$scope.activeTab = 0;
			}
		};
		this.init();

		//FUNCTIONS
		$scope.LoginCallback = function() {
			$scope.loading = true;
			//logged in, do something. i.e. make api calls to load current tab's data
		};
		$scope.TabChanged = function( i: number ) {
			$scope.activeTab = i;
			// Settings.errors = {};
			$scope.loading = true;
			$scope.saving = false;
			$scope.saved = false;
			$scope.cancelled = false;
			const req: any = {
				method: 'GET',
				url: $scope.promocodeEndpoint,
			};
			switch ( i ) {
				case 0:
					PromoCodesService.query()
						.then( ( response: any ) => {
							$scope.promoCodes = response.promoCodes;
							$scope.loading = false;
						} );
					break;
				case 1:
					$scope.loading = false;
					break;
				case 2:
					req.url += $scope.selectedPromocode.promo_code_id;
					req.params = $scope.searchProductParams;

					PromoCodesService.get( $scope.selectedPromocode.promo_code_id )
						.then( ( response: any ) => {
							$scope.selectedPromocode = response.promoCode;
							$scope.selectedPromocode.auto_add = $scope.selectedPromocode.auto_add ? true : false;
							$scope.selectedPromocode.date_effective = $scope.selectedPromocode.date_effective ? ( new Date( $scope.selectedPromocode.date_effective ) ) : null;
							$scope.selectedPromocode.date_expires = $scope.selectedPromocode.date_expires ? ( new Date( $scope.selectedPromocode.date_expires ) ) : null;
							$scope.selectedPromocode.require_all_items = $scope.selectedPromocode.require_all_items ? true : false;
							$scope.selectedPromocode.customerName = response.promoCode.customer ? response.promoCode.customer.name : '';
							$scope.loading = false;
						} );
					break;
			}
		};

		$scope.AddPromoCode = function() {
			$scope.loading = true;
			if ( $scope.addedProducts.length ) {
				$scope.addParams.products = $scope.addedProducts.map( function( product: any ) {
					return { product_id: product.product_id };
				} );
			}
			const data = $scope.addParams;
			PromoCodesService.save( data )
				.then( ( response: any ) => {
					if ( ! response.errors.length && response.promoCode.promo_code_id ) {
						$scope.selectedPromocode = {
							promo_code_id: response.promo_code_id,
						};
						$scope.activeTab = 2;
						$scope.addParams = {
							auto_add: false,
							require_all_items: false,
							products: {
								data: [],
							},
							discount_type: $scope.discountTypes.percentage,
						};
						$scope.addedProducts = [];
					}
					$scope.loading = false;
				} );
		};
		$scope.ChangeTab = function( tabIndex: number, promocodeIndex: number ) {
			$scope.activeTab = tabIndex;
			$scope.selectedPromocode = $scope.promoCodes.data[ promocodeIndex ];
		};
		$scope.CustomerSearchResults = function( resp: any ) { //callback that is used by the search modal
			if ( resp.length ) {
				if ( $scope.activeTab === 1 ) {
					$scope.addParams.customer_id = resp[ 0 ].value.id;
					$scope.addParams.customerName = resp[ 0 ].value.name;
					$scope.addParams.email = resp[ 0 ].value.email;
				} else {
					$scope.selectedPromocode.customer_id = resp[ 0 ].value.id;
					$scope.selectedPromocode.customerName = resp[ 0 ].value.name;
					$scope.selectedPromocode.email = resp[ 0 ].value.email;
				}
			}
		};

		this.deletePromoCode = () => {
			this.isDeleting = true;
			const id = $scope.selectedPromocode.promo_code_id;
			PromoCodesService.delete( id )
				.then( ( ) => {
					this.isDeleting = false;
					$scope.activeTab = 0;
					$scope.selectedPromocode = null;
				} );
		};

		$scope.OpenAddSearchModal = function( type: 'customer' | 'product' ) {
			const callback: any = {};
			let data = {};
			if ( type === 'customer' ) {
				data = {
					tabs: [
						{
							http: {
								method: 'GET',
								url: $scope.customerEndpoint,
								params: {
									q: '',
								},
							},
							relationships: [ 'customers' ],
							fields: [ 'id', 'name', 'email', 'status' ],
							fieldPrefixes: [ 'ID: ', 'Name: ', 'Email: ', 'Status: ' ],
							id: 'id',
							selectOne: true,
						},
					],
				};
				callback.confirm = $scope.CustomerSearchResults;
			} else if ( type === 'product' ) {
				data = {
					tabs: [
						{
							http: {
								method: 'GET',
								url: $scope.productEndpoint,
								params: {
									q: '',
								},
							},
							relationships: [ 'products' ],
							fields: [ 'sku', 'image', 'description', 'status' ],
							fieldPrefixes: [ '', '', '', 'Status: ' ],
							id: 'product_id',
							selectOne: false,
						},
					],
				};
				callback.confirm = $scope.ProductSearchResults;
			}
			callback.cancel = function() {
				angular.noop();
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.addParams.code || 'New Promo Code', data, 'Add Customer' );
		};
		$scope.OpenUpdateSearchModal = function( type: 'customer' | 'product' ) {
			const callback: any = {};
			let data = {};
			if ( type === 'customer' ) {
				data = {
					tabs: [
						{
							http: {
								method: 'GET',
								url: $scope.customerEndpoint,
								params: {
									q: '',
								},
							},
							relationships: [ 'customers' ],
							fields: [ 'id', 'name', 'email', 'status' ],
							fieldPrefixes: [ 'ID: ', 'Name: ', 'Email: ', 'Status: ' ],
							id: 'id',
							selectOne: true,
						},
					],
				};
				callback.confirm = $scope.CustomerSearchResults;
			} else if ( type === 'product' ) {
				data = {
					tabs: [
						{
							http: {
								method: 'GET',
								url: $scope.productEndpoint,
								params: {
									q: '',
								},
							},
							relationships: [ 'products' ],
							fields: [ 'sku', 'image', 'description', 'status' ],
							fieldPrefixes: [ '', '', '', 'Status: ' ],
							id: 'product_id',
							selectOne: false,
						},
						{
							http: {
								method: 'GET',
								url: $scope.promocodeEndpoint + $scope.selectedPromocode.promo_code_id,
								params: {
									q: '',
								},
							},
							relationships: [ 'promoCode', 'products' ],
							fields: [ 'sku', 'image', 'description', 'status' ],
							fieldPrefixes: [ '', '', '', 'Status: ' ],
							id: 'product_id',
						},
					],
				};
				callback.confirm = $scope.ProductSearchResults;
			}
			callback.cancel = function() {
				angular.noop();
			};
			//callback object (confirm, cancel), title, message, optionCancel, optionConfirm, httpConfig
			SearchModal.Show( callback, $scope.selectedPromocode.code, data, 'Add Items' );
		};
		$scope.ProductSearchResults = function( resp: any ) {
			if ( resp.length ) {
				if ( $scope.activeTab === 1 ) {
					if ( $scope.addParams.products && $scope.addParams.products.length ) {
						const currentIds = $scope.addParams.products.map( function( product: any ) {
							return product.product_id;
						} );
						for ( let i = 0; i < resp.length; i++ ) {
							if ( ! currentIds.includes( resp[ i ].value.product_id ) ) {
								$scope.addedProducts.push( resp[ i ].value );
							}
						}
					} else {
						resp.forEach( function( item: any ) {
							$scope.addedProducts.push( item.value );
						} );
					}
				} else {
					resp.forEach( function( item: any ) {
						const currentIds = $scope.selectedPromocode.products.data.map( function( product: any ) {
							return product.product_id;
						} );
						const updatedIds = $scope.updatedProducts.map( function( product: any ) {
							return product.product_id;
						} );
						if ( item.action === 'add' && ! currentIds.includes( item.value.product_id ) ) {
							$scope.updatedProducts.push( item );
						}
						if ( item.action === 'remove' && ! updatedIds.includes( item.value.product_id ) ) {
							$scope.updatedProducts.push( item );
						}
					} );
				}
			}
		};

		$scope.SearchProducts = function( page: number ) {
			$scope.loadingMore = true;
			$scope.searchTerm = $scope.searchProductParams.q;
			$scope.searchProductParams.page = page || 1;
			const req = {
				method: 'GET',
				url: $scope.promocodeEndpoint + $scope.selectedPromocode.promo_code_id,
				params: $scope.searchProductParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.selectedPromocode.products = resp.products;
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.loadingMore = false;
			} );
		};

		this.searchPromoCodes = ( ) => {
			$scope.loadingMore = true;
			const req = {
				method: 'GET',
				url: $scope.promocodeEndpoint,
				params: vm.searchParams,
			};

			PromoCodesService.query( this.searchParams )
				.then( ( response: any ) => {
					$scope.promoCodes = response.promoCodes;
					$scope.loadingMore = false;
				} );
		};

		this.setPage = ( page: number ) => {
			this.searchParams.page = page;
			this.searchPromoCodes();
		};

		this.setRpp = ( rpp: number ) => {
			this.searchParams.rpp = rpp;
			this.searchPromoCodes();
		};

		this.setQuery = ( query: string ) => {
			this.searchParams.page = 1; // Reset page when query changes.
			this.searchParams.q = query;
			$scope.searchTerm = this.searchParams.q;
			this.searchPromoCodes();
		};

		$scope.ShowChanges = function() {
			$scope.showChanges = ! $scope.showChanges;
		};

		this.changeSortBy = ( sortBy: string, e: Event ) => {
			e.preventDefault();
			vm.sortAscending = ( vm.searchParams.sortBy === sortBy ) ? ! vm.sortAscending : true;
			vm.searchParams.sortBy = sortBy;
			vm.searchParams.sortType = vm.sortAscending ? 'asc' : 'desc';
			this.searchPromoCodes( vm.searchParams.page );
		};

		$scope.UpdatePromoCode = function() {
			$scope.loading = true;
			$scope.selectedPromocode.products = [];
			$scope.selectedPromocode.restricted_products = [];
			angular.forEach( $scope.updatedProducts, function( value ) {
				$scope.selectedPromocode.products.push( value.value.product_id );
			} );
			const id = $scope.selectedPromocode.promo_code_id;
			const data = $scope.selectedPromocode;
			PromoCodesService.update( id, data )
				.then( () => {
					// $scope.TabChanged( 2 );
					$scope.loading = false;
				} );
		};
	}
}() );

