( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'CatalogUomController', CatalogUomController );

	CatalogUomController.$inject = [
		'$scope',
		'$rootScope',
		'$stateParams',
		'Admin',
		'Settings',
		'Debug',
		'Utils',
		'Localized',
		'uom',
	];

	/**
	 * @param {Object} $scope       Angular service
	 * @param {Object} $rootScope   Angular service
	 * @param {Object} $stateParams UI Router service
	 * @param {Object} Admin        VendorFuel service
	 * @param {Object} Settings     VendorFuel service
	 * @param {Object} Debug        VendorFuel service
	 * @param {Object} Utils        VendorFuel service
	 * @param {Object} Localized    VendorFuel service
	 * @param {Object} uom          VendorFuel service
	 */
	function CatalogUomController(
		$scope: any,
		$rootScope: any,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		Settings: any,
		Debug: any,
		Utils: any,
		Localized: any,
		uom: any,
	) {
		/**
		 * Initialization
		 */
		this.init = () => {
			this.breadcrumbs = [
				{ name: 'Catalog', state: 'catalog' },
				{ name: 'UOM', state: 'catalog-uom' },
			];
			$scope.activeTab = parseInt( $stateParams.activeTab );
			$scope.addParams = {};
			$scope.appData = uom.uomData();
			$scope.filledUom = {};
			$scope.isAuthed = Admin.Authed();
			$scope.loading = false;
			$scope.loadingMore = false;
			$scope.rppValues = [ 15, 30, 50, 100 ];
			$scope.saving = false;
			$scope.searchParams = {
				page: 1,
				q: '',
				sortBy: '',
				sortType: '',
				rpp: $scope.rppValues[ 0 ], // Must be defined after rrpValues declared.
			};
			$scope.searchTerm = '';
			$scope.selectedUom = null;
			$scope.uom = {};
			$scope.uomCodes = $scope.appData.uomCodes; // Must be defined after appData declared.
			$scope.uomEndpoint = Localized.apiURL + '/admin/product/uom/';
			$scope.uoms = [];

			//If the view tab is selected, redirect to the search page since selected product is null
			if ( $scope.activeTab === 2 ) {
				$scope.activeTab = 0;
			}
		};
		this.init();

		//FUNCTIONS
		$scope.LoginCallback = function() {
			$scope.loading = true;
			//logged in, do something. i.e. make api calls to load current tab's data
		};
		$scope.TabChanged = function( i: number ) {
			$scope.activeTab = i;
			Settings.errors = {};
			$scope.loading = true;
			const req = {
				method: 'GET',
				url: $scope.uomEndpoint,
			};
			switch ( i ) {
				case 0:
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.uoms = resp.uoms;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
				case 1:
					$scope.loading = false;
					break;
				case 2:
					req.url += $scope.selectedUom.uom_id;
					Utils.getHttpPromise( req ).then( function( resp: any ) {
						$scope.uom = resp.uom;
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
					break;
			}
		};
		$scope.AddUom = function() {
			$scope.saving = true;
			const req = {
				method: 'POST',
				url: $scope.uomEndpoint,
				data: $scope.addParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.addParams = {};
				if ( ! resp.errors.length ) {
					$scope.selectedUom = {
						uom_id: resp.uom_id,
					};
					$scope.TabChanged( 2 );
				}
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.saving = false;
			} );
		};
		$scope.ChangeTab = function( tabIndex: number, uomIndex: number ) {
			$scope.activeTab = tabIndex;
			$scope.selectedUom = $scope.uoms.data[ uomIndex ];
		};
		// eslint-disable-next-line no-shadow
		$scope.FillUom = function( uom: any ) {
			$scope.filledUom = uom;
			$scope.addParams.uom = uom.name;
			$scope.addParams.description = uom.name;
			$scope.addParams.code = uom.val;
		};
		$scope.DeleteUom = function() {
			$scope.saving = true;
			const req = {
				method: 'DELETE',
				url: $scope.uomEndpoint + $scope.uom.uom_id,
			};
			Utils.getHttpPromise( req ).then( function() {
				$scope.selectedUom = null;
				$scope.TabChanged( 0 );
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.saving = false;
				Localized.setNotification( {
					type: 'success',
					message: 'Uom Successfully Updates',
				} );
			} );
		};
		$scope.SearchCancelled = function() { //callback that is used by the search modal

		};
		$scope.SearchResults = function( resp: any ) { //callback that is used by the search modal
			Debug.log( resp );
		};

		$scope.searchUoms = function() {
			$scope.loadingMore = true;
			const req = {
				method: 'GET',
				url: $scope.uomEndpoint,
				params: $scope.searchParams,
			};
			Utils.getHttpPromise( req )
				.then( ( resp: any ) => {
					$scope.uoms = resp.uoms;
				}, ( errResp: Error ) => {
					Debug.error( errResp );
				} )
				.finally( () => {
					$scope.loadingMore = false;
				} );
		};

		this.setPage = ( page: number ) => {
			$scope.searchParams.page = page;
			$scope.searchUoms();
		};

		this.setRpp = ( rpp: number ) => {
			$scope.searchParams.rpp = rpp;
			$scope.searchUoms();
		};

		this.setQuery = ( query: string ) => {
			$scope.searchParams.page = 1; // Reset page when query changes.
			$scope.searchParams.q = query;
			$scope.searchTerm = $scope.searchParams.q;
			$scope.searchUoms();
		};

		$scope.SortIndex = function( sortBy: string ) {
			$scope.sortAscending = ( $scope.searchParams.sortBy === sortBy ) ? ! $scope.sortAscending : true;
			$scope.searchParams.sortBy = sortBy;
			$scope.searchParams.sortType = $scope.sortAscending ? 'asc' : 'desc';
			$scope.searchUoms();
		};
		$scope.UpdateUom = function() {
			$scope.saving = true;
			const req = {
				method: 'PUT',
				url: $scope.uomEndpoint + $scope.uom.uom_id,
				data: $scope.uom,
			};
			Utils.getHttpPromise( req ).then( function() {
				angular.noop();
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.saving = false;
			} );
		};
	}
}() );

