( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'PunchoutSuppliersController', PunchoutSuppliersController );

	PunchoutSuppliersController.$inject = [
		'$scope',
		'$rootScope',
		'$state',
		'$stateParams',
		'Admin',
		'ConfirmModal',
		'Settings',
		'Debug',
		'Utils',
		'Localized',
	];

	/**
	 * @param {Object} $scope       Angular service
	 * @param {Object} $rootScope   Angular service
	 * @param {Object} $state       UI Router service
	 * @param {Object} $stateParams UI Router service
	 * @param {Object} Admin        VendorFuel service
	 * @param {Object} ConfirmModal VendorFuel service
	 * @param {Object} Settings     VendorFuel service
	 * @param {Object} Debug        VendorFuel service
	 * @param {Object} Utils        VendorFuel service
	 * @param {Object} Localized    VendorFuel service
	 */
	function PunchoutSuppliersController(
		$scope: any,
		$rootScope: any,
		$state: ng.ui.IStateService,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		ConfirmModal: any,
		Settings: any,
		Debug: any,
		Utils: any,
		Localized: any,
	) {
		this.breadcrumbs = [
			{ name: 'Punchout', state: 'punchout' },
			{ name: 'Suppliers', state: 'punchout-suppliers' },
		];

		/**
		 * Sets the active tab based on whether it matches a index of accessible tabs.
		 *
		 * @param {number} tab
		 * @return {number | Object} Sets activeTab to the tab number, or reloads the state to reset the activeTab.
		 */
		const setActiveTab = ( tab: number ) => {
			const index = [ 0, 1 ];
			if ( index.includes( tab ) ) {
				return $scope.activeTab = tab;
			}
			return $state.go( $state.current, { activeTab: 0 }, { reload: true } );
		};

		setActiveTab( parseInt( $stateParams.activeTab ) );
		$scope.addParams = {};
		$scope.addParams.domain_type = 'DUNS';
		$scope.admin = Admin;
		$scope.confirm = ConfirmModal;
		$scope.endpointBase = Localized.apiURL + '/admin/punchout/supplier/';
		$scope.loading = false;
		$scope.loadingMore = false;
		$scope.localized = Localized;
		$scope.model = {};
		$scope.models = [];
		$scope.rppValues = [ 15, 30, 50, 100 ];
		$scope.saved = false;
		$scope.saving = false;
		$scope.searchParams = {
			q: '',
			sortBy: '',
			sortType: 'asc',
			rpp: $scope.rppValues[ 0 ],
		};
		$scope.settings = Settings;
		$scope.utils = Utils;

		//FUNCTIONS
		this.selectSupplier = ( id: number ) => {
			$scope.model.id = id;
			$scope.activeTab = 2;
		};

		$scope.LoginCallback = function() {
			$scope.loading = true;
		};

		$scope.SortIndex = function( sortBy: string ) {
			$scope.sortAscending = ( $scope.searchParams.sortBy === sortBy ) ? ! $scope.sortAscending : true;
			$scope.searchParams.sortBy = sortBy;
			$scope.searchParams.sortType = $scope.sortAscending ? 'asc' : 'desc';
			$scope.Index( $scope.searchParams.page );
		};

		$scope.TabChanged = function( i: number ) {
			$scope.activeTab = i;

			switch ( $scope.activeTab ) {
				case 0:
					if ( ! $scope.models.data ) {
						$scope.Index();
					}
					break;
				case 2:
					if ( $scope.model.id ) {
						$scope.Show( $scope.model.id );
					}
					break;
			}
		};

		//RESTful Functions
		//Index
		$scope.Index = function( page: number ) {
			$scope.loadingMore = true;
			$scope.searchParams.page = page || 1;
			const req = {
				method: 'GET',
				url: $scope.endpointBase,
				params: $scope.searchParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.models = resp.suppliers;
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				$scope.loadingMore = false;
				$scope.loading = false;
			} );
		};

		//Show
		$scope.Show = function( id: number ) {
			$scope.loading = true;
			$scope.model.id = id;
			const req = {
				method: 'GET',
				url: $scope.endpointBase + id,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.model = resp.supplier;
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};

		//Store
		$scope.Store = function() {
			$scope.loading = true;
			const req = {
				method: 'POST',
				url: $scope.endpointBase,
				data: $scope.addParams,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				if ( ! resp.errors.length ) {
					$scope.addParams = {};
					$scope.Show( resp.supplier.id );
					$scope.TabChanged( 2 );
				}
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};

		//Update
		$scope.Update = function( id: number ) {
			$scope.loading = true;
			const req = {
				method: 'PUT',
				url: $scope.endpointBase + id,
				data: $scope.model,
			};
			Utils.getHttpPromise( req ).then( function() {

			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};

		//Delete
		$scope.Delete = function( id: number ) {
			const callback = {
				confirm() {
					$scope.loading = true;
					const req = {
						method: 'DELETE',
						url: $scope.endpointBase + id,
					};
					Utils.getHttpPromise( req ).then( function() {
						$scope.model = {};
						$scope.activeTab = 0;
						$scope.Index();
					}, function( errResp: Error ) {
						Debug.log( errResp );
					} ).finally( function() {
						$scope.loading = false;
					} );
				},
				cancel() { },
			};
			$scope.confirm.Show( callback, 'Delete Supplier?', 'This action cannot be undone.', 'Back', 'DELETE' );
		};

		$scope.AddEndpoint = function( endpoints: any[] ) {
			endpoints.push( {
				transaction: '',
				url: '',
			} );
		};

		$scope.RemoveEndpoint = function( endpoints: any[], index: number ) {
			endpoints.splice( index, 1 );
		};

		$scope.RefreshEndpoints = function( id: number ) {
			$scope.loading = true;
			const req = {
				method: 'GET',
				url: $scope.endpointBase + id + '/endpoints/refresh',
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.model = resp.supplier;
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};
	}
}() );

