( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'ShippingController', ShippingController );

	ShippingController.$inject = [
		'$scope',
		'$rootScope',
		'$stateParams',
		'Admin',
		'Debug',
		'Utils',
		'Localized',
	];

	/**
	 * @param {Object} $scope       Angular service
	 * @param {Object} $rootScope   Angular service
	 * @param {Object} $stateParams UI Router service
	 * @param {Object} Admin        VendorFuel service
	 * @param {Object} Debug        VendorFuel service
	 * @param {Object} Utils        VendorFuel service
	 * @param {Object} Localized    VendorFuel service
	 */
	function ShippingController(
		$scope: any,
		$rootScope: any,
		$stateParams: ng.ui.IStateParamsService,
		Admin: any,
		Debug: any,
		Utils: any,
		Localized: any,
	) {
		/**
		 * Initialization
		 */
		this.init = () => {
			this.breadcrumbs = [
				{ name: 'Shipping', state: 'shipping' },
			];

			$scope.activeTab = parseInt( $stateParams.activeTab ) || 0;
			$scope.isAuthed = Admin.Authed();
			$scope.loading = true;
			$scope.modeEndpoint = Localized.apiURL + '/admin/shipping/mode/';
			$scope.shippingMode = null;
			$scope.updating = false;
		};
		this.init();

		$scope.GetModeSetting = function() {
			const req = {
				method: 'GET',
				url: $scope.modeEndpoint,
			};
			Utils.getHttpPromise( req ).then( function( resp: any ) {
				$scope.shippingMode = resp;
			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.loading = false;
			} );
		};
		$scope.UpdateModeSetting = function() {
			$scope.updating = true;
			const req = {
				method: 'PUT',
				url: $scope.modeEndpoint,
				data: $scope.shippingMode,
			};
			Utils.getHttpPromise( req ).then( function() {

			}, function( errResp: Error ) {
				Debug.error( errResp );
			} ).finally( function() {
				$scope.updating = false;
				Localized.setNotification( {
					type: 'success',
					message: 'Shipping Mode Successfully Updated',
				} );
			} );
		};
	}
}() );

