( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.controller( 'SubscribeController', SubscribeController );

	SubscribeController.$inject = [
		'$rootScope',
		'$scope',
		'$http',
		'$filter',
		'Utils',
		'Settings',
		'Localized',
		'$timeout',
	];

	/**
	 * @param {Object} $rootScope Angular service
	 * @param {Object} $scope     Angular service
	 * @param {Object} $http      Angular service
	 * @param {Object} $filter    Angular service
	 * @param {Object} Utils      VendorFuel service
	 * @param {Object} Settings   VendorFuel service
	 * @param {Object} Localized  VendorFuel service
	 * @param {Object} $timeout   Angular service
	 */
	function SubscribeController(
		$rootScope: any,
		$scope: any,
		$http: ng.IHttpService,
		$filter: ng.IFilterService,
		Utils: any,
		Settings: any,
		Localized: any,
		$timeout: ng.ITimeoutService,
	) {
		/**
		 * Initialization
		 */
		this.init = () => {
			Utils.redirecting = false;

			this.breadcrumbs = [
				{ name: 'Subscribe', state: 'billing' },
			];

			$scope.store = { name: '', url: '', first_name: '', last_name: '', email: '', emailConfirmation: '', password: '', passwordConfirmation: '', phone: '' };

			$scope.billing = { firstName: '', lastName: '', addressLineOne: '', addressLineTwo: '', city: '', state: '', postalCode: '' };
			$scope.formValidation =
				$scope.store.name ||
				$scope.store.url ||
				$scope.store.first_name ||
				$scope.store.last_name ||
				$scope.store.email ||
				$scope.store.emailConfirmation ||
				$scope.store.password ||
				$scope.store.passwordConfirm;
			$scope.orderComplete = false;
			$scope.payment = { cardNumber: '', expMonth: '', expYear: '', cvv: '' };
			$scope.selectedTier = null;
			$scope.signUpDisable = '!store.name || !store.url || !store.first_name || !store.last_name || !store.email || !store.emailConfirmation || !store.password || !store.passwordConfirm ||!terms || !privacy';
			$scope.store = { storeName: '', storeUrl: '', first_name: '', last_name: '', email: '', password: '', passwordConfirm: '' };
			$scope.submittingOrder = false;
			$scope.tiers = [
				{ description: 'Kindling', pricesheets: '1', products: '5', price: 0, value: 4, selected: false },
				{ description: 'Ember', pricesheets: '1', products: '25', price: 19, value: 1, selected: false },
				{ description: 'Inferno', pricesheets: 'Unlimited', products: 'Unlimited', price: 99, value: 3, selected: false },
				{ description: 'Flame', pricesheets: '2', products: '100', price: 49, value: 2, selected: false },
			];
			$scope.vfApiUrl = $rootScope.vfApiUrl;

			Utils.httpGet( Localized.wpRestUrl + '/wp/site/info' )
				.then( function successCallback( response: any ) {
					$scope.store.name = response.name;
					$scope.store.url = response.url;
				} );
		};
		this.init();

		// $scope.cleanErrorMessages = function(str){
		// 	return str.toString().trim()
		// }
		$scope.selectTier = function( tier: any ) {
			$scope.selectedTier = tier;
			angular.forEach( $scope.tiers, function( value ) {
				if ( tier.value === value.value ) {
					value.selected = true;
				} else {
					value.selected = false;
				}
			} );
		};
		$scope.ToggleShowPassword = function( type: any ) {
			if ( type === 'password' ) {
				$scope.showPassword = ! $scope.showPassword;
			}
			if ( type === 'conf' ) {
				$scope.showPasswordConf = ! $scope.showPasswordConf;
			}
		};
		$scope.SignUp = function() {
			$scope.errors = [];
			$scope.submittingOrder = true;
			$scope.orderComplete = false;
			const req = {
				method: 'POST',
				url: Localized.apiURL + '/tenant/stores',

				data: {
					store: {
						tier: $scope.selectedTier.value,
						name: $scope.store.name,
						url: $scope.store.url,
					},
					user: {
						first_name: $scope.store.first_name,
						last_name: $scope.store.last_name,
						email: $scope.store.email,
						email_confirmation: $scope.store.emailConfirmation,
						password: $scope.store.password,
						password_confirmation: $scope.store.passwordConfirm,
						phone: $scope.store.phone,
					},
				},
			};
			$http( req ).then( function( resp: any ) {
				angular.forEach( resp.data.errors, function( value ) {
					Localized.setNotification( {
						type: 'danger',
						message: value,
					} );
				} );
				$scope.storeDetails = resp.data.store;
				$scope.storeDetails.billing = resp.data.billing;
				$timeout( function() {
					if ( resp.data && resp.data.errors && resp.data.errors.length > 0 ) {
						$scope.errors = resp.data.errors;
					} else {
						//TODO: set api_key
						$scope.storeDetails.term = '1 Month';
						$scope.storeDetails.nextBillDate = $filter( 'date' )( $scope.storeDetails.billing.next_payment_due, 'longDate' );
						Localized.api_key = $scope.storeDetails.api_key;
						Settings.general.Get().then( function successCallback() {
							Settings.general.saved.api_key = Localized.api_key;
							Settings.general.Set();
							$scope.orderComplete = true;
						} );
					}
					if ( Utils.ReturnLocation !== null ) {
						Utils.setLocation( Utils.ReturnLocation, false );
						Utils.ReturnLocation = null;
						Utils.redirecting = false;
					}
					$scope.submittingOrder = false;
				}, 1500 );
			}, function( response ) {
				if ( response.data && response.data.errors ) {
					angular.forEach( response.data.errors, function( value ) {
						const err = Utils.cleanErrorMessages( value );
						$scope.errors.push( err );
					} );
				}
				$scope.submittingOrder = false;
				return response;
			} );
		};
	}
}() );

