(function () {
	'use strict';

	angular
		.module('vendorfuelApp')
		.factory('SelectModal', selectModalFactory);

	selectModalFactory.$inject = [
		'$uibModal',
		'Localized',
		'$sce',
	];

	/**
	 * @param {Object} $uibModal UI Bootstrap service
	 * @param {Object} Localized VendorFuel service
	 * @param {Object} $sce Angular service
	 * @return {Object} service
	 */
	function selectModalFactory(
		$uibModal: ng.ui.bootstrap.IModalService,
		Localized: any,
		$sce: ng.ISCEService,
	) {
		const service = {

			/**
			 * @param {any} callback Callback
			 * @param {any} title Modal title
			 * @param {any} optionArr Option array
			 * @param {any} optionCancel Cancel options
			 * @param {any} optionConfirm Confirm options
			 * @param {boolean} backdrop Display backdrop
			 * @param {number} highlightedButton Button index.
			 * @function Show
			 * Takes in an array of options and allows for activating/deactivating each option individually, or en-mass. Each option is an object with the following parameters: name, value, and active. This array of options is returned to the callback function for handling
			 */
			Show: function (callback: any, title: any, optionArr: any, optionCancel: any, optionConfirm: any, backdrop: boolean, highlightedButton: number) {
				optionCancel = optionCancel || 'Cancel';
				optionConfirm = optionConfirm || 'Confirm';
				backdrop = backdrop || true;
				highlightedButton = highlightedButton || 0;
				let optionWidth: number;
				const tempCancelWidth = optionCancel.length;
				const tempConfirmWidth = optionConfirm.length;
				if (tempCancelWidth > tempConfirmWidth) {
					optionWidth = (tempCancelWidth * 9) + 24;
				} else {
					optionWidth = (tempConfirmWidth * 9) + 24;
				}
				let callbackParams = callback.params || {};
				$uibModal.open({
					templateUrl: Localized.root.url + '/assets/app/modals/select.html',
					backdrop,
					controller($scope: any, $uibModalInstance: ng.ui.bootstrap.IModalInstanceService) {
						$scope.modalTitle = $sce.trustAsHtml(title);
						$scope.options = optionArr;
						$scope.optionCancel = optionCancel;
						$scope.optionConfirm = optionConfirm;
						$scope.optionWidth = optionWidth;
						$scope.highlightedButton = highlightedButton;
						/**
						 * Used to filter the options
						 *
						 * @type {string}
						 */
						$scope.filterTerm = '';
						/**
						 * Array of options
						 *
						 * @type {Array}
						 */
						callbackParams = $scope.options;
						/**
						 * @param {Object} row Row
						 * @function OptionFilter
						 * @return {number} Matching
						 * Filter the array of options based on the filterTerm variable and return only matching results
						 */
						$scope.OptionFilter = function (row: any) {
							return (row.name.indexOf($scope.filterTerm || '') !== -1 || row.value.indexOf($scope.filterTerm || '') !== -1);
						};
						/**
						 * @function SelectAll
						 * Activate all options using the MassSelect function
						 */
						$scope.SelectAll = function () {
							massSelect(true);
						};
						/**
						 * @function SelectNone
						 * Deactivate all options using the MassSelect function
						 */
						$scope.SelectNone = function () {
							massSelect(false);
						};
						/**
						 * @param {boolean} value Value
						 * @function MassSelect
						 * Activate/Deactivate all options
						 */
						function massSelect(value: boolean) {
							angular.forEach($scope.options, function (option) {
								option.active = value;
							});
						}
						$scope.Cancel = function () {
							$uibModalInstance.dismiss();
						};
						$scope.Confirm = function () {
							$uibModalInstance.close();
						};
					},
				}).result.then(function () {
					if (callback.confirm) {
						callback.confirm(callbackParams);
					}
				}, function () {
					if (callback.cancel) {
						callback.cancel(callbackParams);
					}
				});
			}
		};
		return service;
	}
}());

