(function () {
	'use strict';

	angular
		.module('vendorfuelApp')
		.factory('TwoFactorModal', twoFactorModalFactory);

	twoFactorModalFactory.$inject = [
		'$uibModal',
		'Localized',
		'$sce',
	];
	/**
	 * @param {Object} $uibModal UI Bootstrap service
	 * @param {Object} Localized VendorFuel service
	 * @param {Object} $sce Angular service
	 * @return {Object} service
	 */
	function twoFactorModalFactory(
		$uibModal: ng.ui.bootstrap.IModalService,
		Localized: any,
		$sce: ng.ISCEService,
	) {
		const service = {
			Show: function (callback: any, title: any, message: any, optionCancel: any, optionConfirm: any, backdrop: boolean, highlightedButton: number) {
				message = message || '';
				optionCancel = optionCancel || 'Cancel';
				optionConfirm = optionConfirm || 'Confirm';
				backdrop = backdrop || true;
				highlightedButton = highlightedButton || 0;
				let optionWidth: number;
				const tempCancelWidth = optionCancel.length;
				const tempConfirmWidth = optionConfirm.length;
				if (tempCancelWidth > tempConfirmWidth) {
					optionWidth = (tempCancelWidth * 9) + 24;
				} else {
					optionWidth = (tempConfirmWidth * 9) + 24;
				}
				let callbackParams = callback.params || {};
				$uibModal.open({
					templateUrl: Localized.root.url + '/assets/app/modals/two-factor.html',
					backdrop,
					controller($scope: any, $uibModalInstance: ng.ui.bootstrap.IModalInstanceService) {
						$scope.callbackParams = callbackParams;
						$scope.modalTitle = $sce.trustAsHtml(title);
						$scope.modalMessage = $sce.trustAsHtml(message);
						$scope.modalImg = callbackParams.imgSrc;
						$scope.optionCancel = optionCancel;
						$scope.optionConfirm = optionConfirm;
						$scope.optionWidth = optionWidth;
						$scope.highlightedButton = highlightedButton;
						$scope.Cancel = function () {
							$uibModalInstance.dismiss();
						};
						$scope.Confirm = function () {
							$uibModalInstance.close();
						};
						$scope.ApplyChangeToCallback = function () {
							callbackParams = $scope.callbackParams;
						};
					},
				}).result.then(function () {
					if (callback.confirm) {
						callback.confirm(callbackParams);
					}
				}, function () {
					if (callback.cancel) {
						callback.cancel(callbackParams);
					}
				});
			}

		};
		return service;
	}
}());

