( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.factory( 'Reports', reportsFactory );

	/**
	 * @return {Object} Service
	 */
	function reportsFactory() {
		const storeTables = {
			admin_groups: [
				{ name: 'Group Id', value: 'admin_groups.group_id', table: 'admin_groups' },
				{ name: 'Type', value: 'admin_groups.type', table: 'admin_groups' },
				{ name: 'Group', value: 'admin_groups.group', table: 'admin_groups' },
				{ name: 'Description', value: 'admin_groups.description', table: 'admin_groups' },
				{ name: 'Users', value: 'admin_groups.users', table: 'admin_groups' },
			],
			admin_users: [
				{ name: 'Admin User Id', value: 'admin_users.id', table: 'admin_users' },
				{ name: 'Name', value: 'admin_users.name', table: 'admin_users' },
				{ name: 'Email', value: 'admin_users.email', table: 'admin_users' },
				{ name: 'Last Login', value: 'admin_users.last_login_at', table: 'admin_users' },
				{ name: 'Created', value: 'admin_users.created_at', table: 'admin_users' },
			],
			banner_areas: [
				{ name: 'Area Id', value: 'banner_areas.area_id', table: 'banner_areas' },
				{ name: 'Name', value: 'banner_areas.name', table: 'banner_areas' },
				{ name: 'Description', value: 'banner_areas.description', table: 'banner_areas' },
			],
			categories: [
				{ name: 'Category Id', value: 'categories.cat_id', table: 'categories' },
				{ name: 'Parent Id', value: 'categories.parent_id', table: 'categories' },
				{ name: 'Title', value: 'categories.title', table: 'categories' },
				{ name: 'UNSPSC', value: 'categories.unspsc', table: 'categories' },
				{ name: 'Slug', value: 'categories.slug', table: 'categories' },
			],
			customers: [
				{ name: 'Name', value: 'customers.name', table: 'customers' },
				{ name: 'Customer Id', value: 'customers.id', table: 'customers' },
				{ name: 'Email', value: 'customers.email', table: 'customers' },
				{ name: 'Punchout Identity', value: 'customers.punchout_identity', table: 'customers' },
				{ name: 'Shared Secret', value: 'customers.punchout_secret', table: 'customers' },
				{ name: 'Terms', value: 'customers.terms', table: 'customers' },
				{ name: 'Order Prefix', value: 'customers.order_prefix', table: 'customers' },
				{ name: 'Limit Type', value: 'customers.limit_type', table: 'customers' },
				{ name: 'Limit', value: 'customers.limit', table: 'customers' },
				{ name: 'Group ', value: 'customers.group_id', table: 'customers' },
				{ name: 'Price Sheet Id', value: 'customers.price_sheet_id', table: 'customers' },
				{ name: 'Credit Line', value: 'customers.credit_line', table: 'customers' },
				{ name: 'Email Optin', value: 'customers.optin', table: 'customers' },
				{ name: 'Taxable', value: 'customers.taxable', table: 'customers' },
				{ name: 'Punchout Only', value: 'customers.punchout_only', table: 'customers' },
				{ name: 'Lock Billing', value: 'customers.lock_billing', table: 'customers' },
				{ name: 'Lock Shipping', value: 'customers.lock_shipping', table: 'customers' },
				{ name: 'Change Password', value: 'customers.change_pw', table: 'customers' },
				{ name: 'Pending Emails', value: 'customers.pending_emails', table: 'customers' },
			],
			customer_groups: [
				{ name: 'Group Id', value: 'customer_groups.group_id', table: 'customer_groups' },
				{ name: 'Parent Group Id', value: 'customer_groups.parent_group_id', table: 'customer_groups' },
				{ name: 'Name', value: 'customer_groups.name', table: 'customer_groups' },
				{ name: 'Default Price Sheet', value: 'customer_groups.default_price_sheet', table: 'customer_groups' },
				{ name: 'Group Invite Code', value: 'customer_groups.group_invite_code', table: 'customer_groups' },
				{ name: 'Authorized Emails', value: 'customer_groups.authorized_emails', table: 'customer_groups' },
				{ name: 'Group Registration Available', value: 'customer_groups.group_registration_available', table: 'customer_groups' },
				{ name: 'Order Prefix', value: 'customer_groups.order_prefix', table: 'customer_groups' },
				{ name: 'Terms', value: 'customer_groups.terms', table: 'customer_groups' },
				{ name: 'Punchout Identity', value: 'customer_groups.punchout_identity', table: 'customer_groups' },
				{ name: 'Punchout Secret', value: 'customer_groups.punchout_secret', table: 'customer_groups' },
				{ name: 'Shipping Mode', value: 'customer_groups.shipping_mode', table: 'customer_groups' },
			],
			email_templates: [
				{ name: 'Template Id', value: 'email_templates.id', table: 'email_templates' },
				{ name: 'Type', value: 'email_templates.type', table: 'email_templates' },
				{ name: 'Sender Name', value: 'email_templates.sender_name', table: 'email_templates' },
				{ name: 'Sender Email', value: 'email_templates.sender_email', table: 'email_templates' },
				{ name: 'Subject', value: 'email_templates.subject', table: 'email_templates' },
				{ name: 'Message', value: 'email_templates.message', table: 'email_templates' },
				{ name: 'Notification Emails', value: 'email_templates.notification_emails', table: 'email_templates' },
			],
			manufacturers: [
				{ name: 'Manufacturer Id', value: 'manufacturers.id', table: 'manufacturers' },
				{ name: 'Manufacturer', value: 'manufacturers.name', table: 'manufacturers' },
				{ name: 'Info', value: 'manufacturers.info', table: 'manufacturers' },
				{ name: 'Website', value: 'manufacturers.website', table: 'manufacturers' },
			],
			parcels: [
				{ name: 'Parcel Id', value: 'parcels.id', table: 'parcels' },
				{ name: 'Title', value: 'parcels.title', table: 'parcels' },
				{ name: 'Length', value: 'parcels.length', table: 'parcels' },
				{ name: 'Width', value: 'parcels.width', table: 'parcels' },
				{ name: 'Height', value: 'parcels.height', table: 'parcels' },
				{ name: 'Units', value: 'parcels.distance_unit', table: 'parcels' },
			],
			collections: [
				{ name: 'Collection Id', value: 'collections.id', table: 'collections' },
				{ name: 'Name', value: 'collections.name', table: 'collections' },
			],
			order_notifications: [
				{ name: 'Notification Id', value: 'order_notifications.notification_id', table: 'order_notifications' },
				{ name: 'Order Id', value: 'order_notifications.order_id', table: 'order_notifications' },
				{ name: 'Status', value: 'order_notifications.status', table: 'order_notifications' },
				{ name: 'Type', value: 'order_notifications.type', table: 'order_notifications' },
				{ name: 'Recipient', value: 'order_notifications.recipient', table: 'order_notifications' },
				{ name: 'body', value: 'order_notifications.body', table: 'order_notifications' },
				{ name: 'Response', value: 'order_notifications.response', table: 'order_notifications' },
			],
			product_uom: [
				{ name: 'uom Id', value: 'product_uom.uom_id', table: 'product_uom' },
				{ name: 'uom', value: 'product_uom.uom', table: 'product_uom' },
				{ name: 'Code', value: 'product_uom.code', table: 'product_uom' },
				{ name: 'Description', value: 'product_uom.description', table: 'product_uom' },
			],
			price_sheets_index: [
				{ name: 'Price Sheet Id', value: 'price_sheets_index.price_sheet_id', table: 'price_sheets_index' },
				{ name: 'Sheet', value: 'price_sheets_index.sheet', table: 'price_sheets_index' },
				{ name: 'Site Id', value: 'price_sheets_index.site_id', table: 'price_sheets_index' },
				{ name: 'GP Price Sheet', value: 'price_sheets_index.gp_price_sheet', table: 'price_sheets_index' },
			],
			products: [ { name: 'Product Id', value: 'products.product_id', table: 'products' },
				{ name: 'SKU', value: 'products.sku', table: 'products' },
				{ name: 'Slug', value: 'products.slug', table: 'products' },
				{ name: 'UPC', value: 'products.upc', table: 'products' },
				{ name: 'Qty', value: 'products.qty', table: 'products' },
				{ name: 'Manufacturer Id', value: 'products.manufacturer_id', table: 'products' },
				{ name: 'Brand Name', value: 'products.brand_name', table: 'products' },
				{ name: 'Country', value: 'products.country', table: 'products' },
				{ name: 'Device', value: 'products.device', table: 'products' },
				{ name: 'Family', value: 'products.family', table: 'products' },
				{ name: 'Green', value: 'products.green', table: 'products' },
				{ name: 'Green Attributes', value: 'products.green_attributes', table: 'products' },
				{ name: 'Hazmat', value: 'products.hazmat', table: 'products' },
				{ name: 'Includes', value: 'products.includes', table: 'products' },
				{ name: 'Keywords', value: 'products.keywords', table: 'products' },
				{ name: 'Model', value: 'products.model', table: 'products' },
				{ name: 'Msds', value: 'products.msds', table: 'products' },
				{ name: 'Rebate', value: 'products.rebate', table: 'products' },
				{ name: 'Related', value: 'products.related', table: 'products' },
				{ name: 'Alternates', value: 'products.alternates', table: 'products' },
				{ name: 'Truck Only', value: 'products.truck_only', table: 'products' },
				{ name: 'Category', value: 'products.category_id', table: 'products' },
				{ name: 'UoM ID', value: 'products.uomid', table: 'products' },
				{ name: 'UoM Qty', value: 'products.uomqty', table: 'products' },
				{ name: 'UoM Desc', value: 'products.uomdesc', table: 'products' },
				{ name: 'Description', value: 'products.description', table: 'products' },
				{ name: 'Long Description', value: 'products.long_description', table: 'products' },
				{ name: 'Attribute 1 Name', value: 'products.att1n', table: 'products' },
				{ name: 'Attribute 1 Data', value: 'products.att1d', table: 'products' },
				{ name: 'Attribute 2 Name', value: 'products.att2n', table: 'products' },
				{ name: 'Attribute 2 Data', value: 'products.att2d', table: 'products' },
				{ name: 'Attribute 3 Name', value: 'products.att3n', table: 'products' },
				{ name: 'Attribute 3 Data', value: 'products.att3d', table: 'products' },
				{ name: 'Attribute 4 Name', value: 'products.att4n', table: 'products' },
				{ name: 'Attribute 4 Data', value: 'products.att4d', table: 'products' },
				{ name: 'Attribute 5 Name', value: 'products.att5n', table: 'products' },
				{ name: 'Attribute 5 Data', value: 'products.att5d', table: 'products' },
				{ name: 'Attribute 6 Name', value: 'products.att6n', table: 'products' },
				{ name: 'Attribute 6 Data', value: 'products.att6d', table: 'products' },
				{ name: 'Attribute 7 Name', value: 'products.att7n', table: 'products' },
				{ name: 'Attribute 7 Data', value: 'products.att7d', table: 'products' },
				{ name: 'Attribute 8 Name', value: 'products.att8n', table: 'products' },
				{ name: 'Attribute 8 Data', value: 'products.att8d', table: 'products' },
				{ name: 'Attribute 9 Name', value: 'products.att9n', table: 'products' },
				{ name: 'Attribute 9 Data', value: 'products.att9d', table: 'products' },
				{ name: 'Attribute 10 Name', value: 'products.att10n', table: 'products' },
				{ name: 'Attribute 10 Data', value: 'products.att10d', table: 'products' },
				{ name: 'Attribute 11 Name', value: 'products.att11n', table: 'products' },
				{ name: 'Attribute 11 Data', value: 'products.att11d', table: 'products' },
				{ name: 'Attribute 12 Name', value: 'products.att12n', table: 'products' },
				{ name: 'Attribute 12 Data', value: 'products.att12d', table: 'products' },
				{ name: 'Attribute 13 Name', value: 'products.att13n', table: 'products' },
				{ name: 'Attribute 13 Data', value: 'products.att13d', table: 'products' },
				{ name: 'Attribute 14 Name', value: 'products.att14n', table: 'products' },
				{ name: 'Attribute 14 Data', value: 'products.att14d', table: 'products' },
				{ name: 'Attribute 15 Name', value: 'products.att15n', table: 'products' },
				{ name: 'Attribute 15 Data', value: 'products.att15d', table: 'products' },
				{ name: 'Attribute 16 Name', value: 'products.att16n', table: 'products' },
				{ name: 'Attribute 16 Data', value: 'products.att16d', table: 'products' },
				{ name: 'Attribute 17 Name', value: 'products.att17n', table: 'products' },
				{ name: 'Attribute 17 Data', value: 'products.att17d', table: 'products' },
				{ name: 'Attribute 18 Name', value: 'products.att18n', table: 'products' },
				{ name: 'Attribute 18 Data', value: 'products.att18d', table: 'products' },
				{ name: 'Mfg Part #', value: 'products.mfg_part_num', table: 'products' },
				{ name: 'Ability One Sku', value: 'products.ability_one_sku', table: 'products' },
				{ name: 'Status', value: 'products.status', table: 'products' },
				{ name: 'Rating', value: 'products.average_rating', table: 'products' },
			],
			orders: [
				{ name: 'Order Id', value: 'orders.order_id', table: 'orders' },
				{ name: 'Group Id', value: 'orders.group_id', table: 'orders' },
				{ name: 'Customer Id', value: 'orders.customer_id', table: 'orders' },
				{ name: 'Approver Id', value: 'orders.approver_id', table: 'orders' },
				{ name: 'Shipping Id', value: 'orders.shipping_id', table: 'orders' },
				{ name: 'Billing Id', value: 'orders.billing_id', table: 'orders' },
				{ name: 'Cost Center Id', value: 'orders.cost_center_id', table: 'orders' },
				{ name: 'Shipping Rule Id', value: 'orders.shipping_rule_id', table: 'orders' },
				{ name: 'Cost Center Code', value: 'orders.cost_center_code', table: 'orders' },
				{ name: 'Price Sheet', value: 'orders.price_sheet', table: 'orders' },
				{ name: 'Loc Id', value: 'orders.loc_id', table: 'orders' },
				{ name: 'Trx Redeemed', value: 'orders.trx_redeemed', table: 'orders' },
				{ name: 'Status', value: 'orders.status', table: 'orders' },
				{ name: 'Order Shipped', value: 'orders.order_shipped', table: 'orders' },
				{ name: 'Tracking Code', value: 'orders.tracking_code', table: 'orders' },
				{ name: 'Shipping Method', value: 'orders.shipping_method', table: 'orders' },
				{ name: 'Shipping Carrier', value: 'orders.shipping_carrier', table: 'orders' },
				{ name: 'Order Date', value: 'orders.order_date', table: 'orders' },
				{ name: 'Subtotal', value: 'orders.subtotal', table: 'orders' },
				{ name: 'Tax', value: 'orders.tax', table: 'orders' },
				{ name: 'Tax Rate', value: 'orders.tax_rate', table: 'orders' },
				{ name: 'Shipping', value: 'orders.shipping', table: 'orders' },
				{ name: 'Discount', value: 'orders.discount', table: 'orders' },
				{ name: 'Promo Discount', value: 'orders.promo_discount', table: 'orders' },
				{ name: 'Total Amount', value: 'orders.total_amt', table: 'orders' },
				{ name: 'RR PO Num', value: 'orders.rr_po_num', table: 'orders' },
				{ name: 'Issuing Office', value: 'orders.issuing_office', table: 'orders' },
				{ name: 'Approver', value: 'orders.approver', table: 'orders' },
				{ name: 'Attention', value: 'orders.attention', table: 'orders' },
				{ name: 'Organization', value: 'orders.organization', table: 'orders' },
				{ name: 'First Name', value: 'orders.first_name', table: 'orders' },
				{ name: 'Last Name', value: 'orders.last_name', table: 'orders' },
				{ name: 'Email', value: 'orders.email', table: 'orders' },
				{ name: 'Phone', value: 'orders.phone', table: 'orders' },
				{ name: 'Billing First Name', value: 'orders.bill_first_name', table: 'orders' },
				{ name: 'Billing Last Name', value: 'orders.bill_last_name', table: 'orders' },
				{ name: 'Billing Email', value: 'orders.bill_email', table: 'orders' },
				{ name: 'Billing Phone', value: 'orders.bill_phone', table: 'orders' },
				{ name: 'Payment Method', value: 'orders.payment_method', table: 'orders' },
				{ name: 'Credit Line', value: 'orders.credit_line', table: 'orders' },
				{ name: 'Credit Card', value: 'orders.credit_card', table: 'orders' },
				{ name: 'CC Type', value: 'orders.cctype', table: 'orders' },
				{ name: 'Shipping Profile', value: 'orders.shipping_profile', table: 'orders' },
				{ name: 'Address 1', value: 'orders.address1', table: 'orders' },
				{ name: 'Address 2', value: 'orders.address2', table: 'orders' },
				{ name: 'City', value: 'orders.city', table: 'orders' },
				{ name: 'State', value: 'orders.state', table: 'orders' },
				{ name: 'Zip', value: 'orders.zip', table: 'orders' },
				{ name: 'Billing Address 1', value: 'orders.bill_address1', table: 'orders' },
				{ name: 'Billing Address 2', value: 'orders.bill_address2', table: 'orders' },
				{ name: 'Billing City', value: 'orders.bill_city', table: 'orders' },
				{ name: 'Billing State', value: 'orders.bill_state', table: 'orders' },
				{ name: 'Billing Zip', value: 'orders.bill_zip', table: 'orders' },
				{ name: 'Notes', value: 'orders.notes', table: 'orders' },
				{ name: 'GSA Total', value: 'orders.gsa_total', table: 'orders' },
				{ name: 'Other Total', value: 'orders.other_total', table: 'orders' },
				{ name: 'Approver Notes', value: 'orders.approver_notes', table: 'orders' },
				{ name: 'Promo Codes', value: 'orders.promo_codes', table: 'orders' },
				{ name: 'F1 Replace Field', value: 'orders.f1_replace_field', table: 'orders' },
				{ name: 'F1 Name', value: 'orders.f1_name', table: 'orders' },
				{ name: 'F1 XML Location', value: 'orders.f1_xml_location', table: 'orders' },
				{ name: 'F1 Value', value: 'orders.f1_value', table: 'orders' },
				{ name: 'F2 Replace Field', value: 'orders.f2_replace_field', table: 'orders' },
				{ name: 'F2 Name', value: 'orders.f2_name', table: 'orders' },
				{ name: 'F2 XML Location', value: 'orders.f2_xml_location', table: 'orders' },
				{ name: 'F2 Value', value: 'orders.f2_value', table: 'orders' },
				{ name: 'F3 Name', value: 'orders.f3_name', table: 'orders' },
				{ name: 'F3 Value', value: 'orders.f3_name', table: 'orders' },
				{ name: 'F4 Name', value: 'orders.f4_name', table: 'orders' },
				{ name: 'F4 Value', value: 'orders.f4_name', table: 'orders' },
				{ name: 'F5 Name', value: 'orders.f5_name', table: 'orders' },
				{ name: 'F5 Value', value: 'orders.f5_name', table: 'orders' },
				{ name: 'F6 Name', value: 'orders.f6_name', table: 'orders' },
				{ name: 'F6 Value', value: 'orders.f6_name', table: 'orders' },
				{ name: 'Reference Id', value: 'orders.reference_id', table: 'orders' },
				{ name: 'Payload Id', value: 'orders.payload_id', table: 'orders' },
				{ name: 'Postback Url', value: 'orders.postback_url', table: 'orders' },
				{ name: 'Shipping Token', value: 'orders.shipping_token', table: 'orders' },
			],
		};
		const joinTables = {
			group_admins: [
				{ name: 'Customer ID', value: 'group_admins.customer_id', table: 'group_admins' },
				{ name: 'Group ID', value: 'group_admins.group_id', table: 'group_admins' },
			],

			group_approvers: [
				{ name: 'Customer ID', value: 'group_approvers.customer_id', table: 'group_approvers' },
				{ name: 'Group ID', value: 'group_approvers.group_id', table: 'group_approvers' },
			],

			inventory_logs: [
				{ name: 'Id', value: 'inventory_logs.id', table: 'inventory_logs' },
				{ name: 'Product Id', value: 'inventory_logs.product_id', table: 'inventory_logs' },
				{ name: 'In', value: 'inventory_logs.inventory_in', table: 'inventory_logs' },
				{ name: 'Out', value: 'inventory_logs.inventory_out', table: 'inventory_logs' },
				{ name: 'Date', value: 'inventory_logs.created_at', table: 'inventory_logs' },
			],

			pending_items: [
				{ name: 'Id', value: 'pending_items.pend_id', table: 'pending_items' },
				{ name: 'Cart Items Id', value: 'pending_items.cart_item_id', table: 'pending_items' },
				{ name: 'Order Id', value: 'pending_items.order_id', table: 'pending_items' },
				{ name: 'Product Id', value: 'pending_items.product_id', table: 'pending_items' },
				{ name: 'Qty', value: 'pending_items.qty', table: 'pending_items' },
				{ name: 'Price', value: 'pending_items.price', table: 'pending_items' },
			],

			price_sheets: [
				{ name: 'Id', value: 'price_sheets.ps_item_id', table: 'price_sheets' },
				{ name: 'Price Sheet Id', value: 'price_sheets.price_sheet_id', table: 'price_sheets' },
				{ name: 'Product Id', value: 'price_sheets.product_id', table: 'price_sheets' },
				{ name: 'Sku', value: 'price_sheets.sku', table: 'price_sheets' },
				{ name: 'GSA', value: 'price_sheets.gsa', table: 'price_sheets' },
				{ name: 'Ability One', value: 'price_sheets.ability_one', table: 'price_sheets' },
				{ name: 'Core List', value: 'price_sheets.core_list', table: 'price_sheets' },
				{ name: 'Price', value: 'price_sheets.price', table: 'price_sheets' },
			],

			product_collection: [
				{ name: 'Collection Id', value: 'product_collection.collection_id', table: 'product_collection' },
				{ name: 'Product ID', value: 'product_collection.product_id', table: 'product_collection' },
			],

			purchased_items: [
				{ name: 'Purchase Id', value: 'purchased_items.purch_id', table: 'purchased_items' },
				{ name: 'Customer Id', value: 'purchased_items.customer_id', table: 'purchased_items' },
				{ name: 'Order Id', value: 'purchased_items.order_id', table: 'purchased_items' },
				{ name: 'Product Id', value: 'purchased_items.product_id', table: 'purchased_items' },
				{ name: 'Purchase Date', value: 'purchased_items.purchase_date', table: 'purchased_items' },
				{ name: 'Quantity', value: 'purchased_items.qty', table: 'purchased_items' },
				{ name: 'Price', value: 'purchased_items.price', table: 'purchased_items' },
				{ name: 'GSA', value: 'purchased_items.gsa', table: 'purchased_items' },
				{ name: 'Ability One', value: 'purchased_items.ability_one', table: 'purchased_items' },
				{ name: 'Core List', value: 'purchased_items.core_list', table: 'purchased_items' },
				{ name: 'Return Status', value: 'purchased_items.return_status', table: 'purchased_items' },
				{ name: 'Return Request Date', value: 'purchased_items.return_request_date', table: 'purchased_items' },
				{ name: 'Return Respone Notes', value: 'purchased_items.return_response_notes', table: 'purchased_items' },
				{ name: 'Return Reason', value: 'purchased_items.return_reason', table: 'purchased_items' },
				{ name: 'Return Qty', value: 'purchased_items.return_qty', table: 'purchased_items' },
				{ name: 'Return Contact', value: 'purchased_items.return_contact', table: 'purchased_items' },
				{ name: 'Return Boxes', value: 'purchased_items.return_boxes', table: 'purchased_items' },
			],

			saved_billing: [
				{ name: 'Id', value: 'saved_billing.billing_id', table: 'saved_billing' },
				{ name: 'Customer Id', value: 'saved_billing.customer_id', table: 'saved_billing' },
				{ name: 'Name', value: 'saved_billing.billing_name', table: 'saved_billing' },
				{ name: 'First Name', value: 'saved_billing.first_name', table: 'saved_billing' },
				{ name: 'Last Name', value: 'saved_billing.last_name', table: 'saved_billing' },
				{ name: 'Email', value: 'saved_billing.email', table: 'saved_billing' },
				{ name: 'Address 1', value: 'saved_billing.address1', table: 'saved_billing' },
				{ name: 'Address 2', value: 'saved_billing.address2', table: 'saved_billing' },
				{ name: 'City', value: 'saved_billing.city', table: 'saved_billing' },
				{ name: 'State', value: 'saved_billing.state', table: 'saved_billing' },
				{ name: 'Zip', value: 'saved_billing.zip', table: 'saved_billing' },
				{ name: 'Area Code', value: 'saved_billing.areacode', table: 'saved_billing' },
				{ name: 'Phone', value: 'saved_billing.phone', table: 'saved_billing' },
				{ name: 'Active', value: 'saved_billing.active', table: 'saved_billing' },
			],

			saved_shipping: [
				{ name: 'Id', value: 'saved_shipping.shipping_id', table: 'saved_shipping' },
				{ name: 'Customer Id', value: 'saved_shipping.customer_id', table: 'saved_shipping' },
				{ name: 'Name', value: 'saved_shipping.shipping_name', table: 'saved_shipping' },
				{ name: 'First Name', value: 'saved_shipping.first_name', table: 'saved_shipping' },
				{ name: 'Last Name', value: 'saved_shipping.last_name', table: 'saved_shipping' },
				{ name: 'Email', value: 'saved_shipping.email', table: 'saved_shipping' },
				{ name: 'Address 1', value: 'saved_shipping.address1', table: 'saved_shipping' },
				{ name: 'Address 2', value: 'saved_shipping.address2', table: 'saved_shipping' },
				{ name: 'City', value: 'saved_shipping.city', table: 'saved_shipping' },
				{ name: 'State', value: 'saved_shipping.state', table: 'saved_shipping' },
				{ name: 'Zip', value: 'saved_shipping.zip', table: 'saved_shipping' },
				{ name: 'Area Code', value: 'saved_shipping.areacode', table: 'saved_shipping' },
				{ name: 'Phone', value: 'saved_shipping.phone', table: 'saved_shipping' },
				{ name: 'Active', value: 'saved_shipping.active', table: 'saved_shipping' },
			],

		};
		let allTables: any = {};

		const service = {
			getStoreTables() {
				return storeTables;
			},
			getAllTables() {
				return allTables;
			},
			getComparators() {
				return [ 'contains', '!contains', '=', '!=', '>', '<', 'begins', '!begins', 'ends', '!ends' ];
			},
		};

		allTables = Object.assign( {}, storeTables );
		allTables.inventory_logs = joinTables.inventory_logs;
		allTables.group_admins = joinTables.group_admins;
		allTables.group_approvers = joinTables.group_approvers;
		allTables.pending_items = joinTables.pending_items;
		allTables.price_sheets = joinTables.price_sheets;
		allTables.purchased_items = joinTables.purchased_items;
		allTables.saved_billing = joinTables.saved_billing;
		allTables.saved_shipping = joinTables.saved_shipping;

		return service;
	}
}() );

