(function () {
	'use strict';

	angular
		.module('vendorfuelApp')
		.factory('Settings', settingsFactory);

	settingsFactory.$inject = [
		'$timeout',
		'$window',
		'Localized',
		'Utils',
		'Admin',
	];

	/**
	 * @param {Object} $timeout Angular service
	 * @param {Object} $window Angular service
	 * @param {Object} Localized VendorFuel service
	 * @param {Object} Utils VendorFuel service
	 * @param {Object} Admin VendorFuel service
	 * @return {Object} Service
	 */
	function settingsFactory(
		$timeout: ng.ITimeoutService,
		$window: ng.IWindowService,
		Localized: any,
		Utils: any,
		Admin: any,
	) {

		/**
		 * @param {any} tab Tab
		 * @param {string} url URL
		 * @return {Object} Promise
		 */
		function getSettings(tab: any, url?: string) {
			let httpUrl = Localized.apiURL + url;
			if (!url) {
				httpUrl = Localized.wpRestUrl + '/settings/' + tab.name;
			}
			const promise = Utils.httpGet(httpUrl).then(function successCallback(response: any) {
				tab.saved = response;
			},
				function errorCallback(response: any) {
					tab.saved = response;
				});
			return promise;
		}
		/**
		 * @param {any} tab Tab
		 * @param {Object} params Params
		 * @param {string} url URL
		 * @param {Object} data Data
		 * @return {Object} Promise
		 */
		function postSettings(tab: any, params: any, url?: string, data?: any) {
			let httpUrl = Localized.apiURL + url;
			if (!url) {
				httpUrl = Localized.wpRestUrl + '/settings/' + tab.name;
			}
			const promise = Utils.httpPost(httpUrl,
				params, data).then(function successCallback(response: any) {
					if (response.errors) {
						service.errors = response.errors;
					}
				},
					function errorCallback(response: any) {
						if (response.errors) {
							service.errors = response.errors;
						}
					});
			return promise;
		}
		/**
		 * @param {any} tab Tab
		 * @param {Object} params Params
		 * @param {string} url URL
		 * @param {Object} data Data
		 * @return {Object} Promise
		 */
		function putSettings(tab: any, params: any, url: string, data?: any) {
			let httpUrl = Localized.apiURL + url;
			if (!url) {
				httpUrl = Localized.wpRestUrl + '/settings/' + tab.name + '/post';
			}
			const promise = Utils.httpPut(httpUrl,
				params, data).then(function successCallback(response: any) {
					if (response.errors) {
						service.errors = response.errors;
					}
				},
					function errorCallback(response: any) {
						if (response.errors) {
							service.errors = response.errors;
						}
					});
			return promise;
		}

		const service = {
			errors: {},
			resetTutorial,
			general: {
				name: 'general',
				saved: {} as any,
				Get() {
					return getSettings(service.general);
				},
				Set() {
					const params = {
						settings: JSON.stringify(service.general.saved),
					};
					return postSettings(service.general, params).then(function () {
						if (service.general.saved.api_url !== Localized.apiURL) {
							Localized.apiURL = service.general.saved.api_url;
							Admin.Logout();
						}
						if (service.general.saved.api_key !== Localized.api_key) {
							Localized.api_key = service.general.saved.api_key;
							Admin.Logout();
						}
					});
				},
			},
			store: {
				name: 'store',
				saved: {} as any,
				Get() {
					return getSettings(service.store, '/stores').then(function () {
						service.store.saved = service.store.saved.store;
					});
				},
				Set() {
					const params = {
						name: service.store.saved.name,
						url: service.store.saved.url,
						default_order_prefix: service.store.saved.default_order_prefix,
						default_customer_prefix: service.store.saved.default_customer_prefix,
						options: service.store.saved.options,
						checkout: service.store.saved.checkout,
					};
					postSettings(service.store, params);
					return putSettings(service.store, null, '/stores', params);
				},
				Changed() {
				},
			},
			analytics: {
				name: 'analytics',
				saved: {} as any,
				Get() {
					return getSettings(service.analytics);
				},
				Set() {
					const params = {
						settings: JSON.stringify(service.analytics.saved),
					};
					return postSettings(service.analytics, params);
				},
				Changed(enabled: any) {
					if (!enabled) {
						service.analytics.saved.google.id = '';
						service.analytics.saved.google.label = '';
						service.analytics.saved.google.tracking = false;
					}
				},
			},
			image: {
				name: 'image',
				saved: {},
				Get() {
					return getSettings(service.image).then(function () {
						service.image.ChangeRepo();
					});
				},
				Set() {
					const params = {
						settings: JSON.stringify(service.image.saved),
					};
					return postSettings(service.image, params);
				},
				repositories: [{
					name: 'Local',
					value: 0,
				},
				{
					name: 'Amazon S3',
					value: 1,
				},
				{
					name: 'MaxCDN',
					value: 2,
				},
				],
				apiKeyRequired: false,
				Changed() {
					service.image.saved.storage.repository = service.image.saved.imageRepository.value;
					if (service.image.saved.imageRepository.value === 1) {
						service.image.apiKeyRequired = true;
					} else {
						service.image.apiKeyRequired = false;
						service.image.saved.storage.api_key = '';
					}
				},
			} as any,
			anguledit: {
				name: 'anguledit',
				saved: {},
				Get() {
					return getSettings(service.anguledit);
				},
				Set() {
					const params = {
						settings: JSON.stringify(service.anguledit.saved),
					};
					return postSettings(service.anguledit, params).then(function () {

					});
				},
			},
			gateways: {
				name: 'gateways',
				saved: {} as any,
				Get() {
					return getSettings(service.gateways, '/admin/gateways').then(function () {
						service.gateways.saved.gateways.authnet.enabled = service.gateways.saved.gateways.authnet.enabled === 'true';
						service.gateways.saved.gateways.paypal.enabled = service.gateways.saved.gateways.paypal.enabled === 'true';
						service.gateways.Changed(0, service.gateways.saved.gateways.authnet.enabled);
						service.gateways.Changed(1, service.gateways.saved.gateways.paypal.enabled);
					});
				},
				Set() {
					const params = {
						'gateways[authnet][enabled]': service.gateways.saved.gateways.authnet.enabled,
						'gateways[authnet][id]': service.gateways.saved.gateways.authnet.id,
						'gateways[authnet][key]': service.gateways.saved.gateways.authnet.key,
						'gateways[paypal][enabled]': service.gateways.saved.gateways.paypal.enabled,
						'gateways[paypal][client_id]': service.gateways.saved.gateways.paypal.client_id,
						'gateways[paypal][secret]': service.gateways.saved.gateways.paypal.secret,
					};
					return putSettings(service.gateways, params, '/admin/gateways');
				},
				Changed(i: number, enabled: boolean) {
					if (!enabled && i === 0) {
						service.gateways.saved.gateways.authnet.id = '';
						service.gateways.saved.gateways.authnet.key = '';
					} else if (!enabled && i === 1) {
						service.gateways.saved.gateways.paypal.client_id = '';
						service.gateways.saved.gateways.paypal.secret = '';
					}
				},
			},
			billing: {
				name: 'billing',
				saved: {} as any,
				Get() {
					return getSettings(service.billing,
						'/tenant/billing/check');
				},
				Set() {
					const params = {
						firstName: service.billing.saved.first_name,
						lastName: service.billing.saved.last_name,
						number: service.billing.saved.card.number,
						expiryMonth: service.billing.saved.card.expMonth,
						expiryYear: service.billing.saved.card.expYear,
						cvv: service.billing.saved.card.cvv,
					};
					return postSettings(service.billing, params, '/tenant/billing/update');
				},
				Cancel() {
					const params = {
						store: service.billing.saved.store.id,
					};
					return postSettings(service.billing, params, '/tenant/billing/cancel');
				},
				cancelling: false,
			}
		};
		return service;

		/**
		 * @return {any} Timeout
		 */
		function resetTutorial() {
			return $timeout(function () {
				$window.localStorage.setItem('tutorialComplete', 'false');
				$window.localStorage.setItem('showTutorial', 'true');
			}, 1000)
				.then(function () {
					Localized.setNotification({
						type: 'success',
						message: 'Tutorial cookies have been reset.',
					});
				});
		}
	}
}());

