(function () {
	'use strict';

	angular
		.module('vendorfuelApp')
		.factory('Tenant', tenantFactory);

	tenantFactory.$inject = [
		'$http',
		'$cookies',
		'Localized',
		'Debug',
	];

	/**
	 * @param {Object} $http Angular service
	 * @param {Object} $cookies Angular service
	 * @param {Object} Localized VendorFuel service
	 * @param {Object} Debug VendorFuel service
	 * @return {Object} Service
	 */
	function tenantFactory(
		$http: ng.IHttpService,
		$cookies: ng.cookies.ICookiesService,
		Localized: any,
		Debug: any,
	) {
		const service = {
			login_errors: {} as any,
			Login: function (e: string, p: string) { //TODO: get api url from settings factory
				const req = {
					method: 'POST',
					url: Localized.apiURL + '/tenant/login',
					data: {
						email: e,
						password: p,
					},
				};
				const promise = $http(req).then(function successCallback(response: any) {
					if (Object.keys(response.data.errors).length > 0) {
						Debug.error('Login errors: ', response.data.errors);
						service.login_errors = response.data.errors;
					} else {
						tenant.name = response.data.name;
						$cookies.put('vendorfuel-tenant-name', tenant.name);
						tenant.tokens.jwt = response.data.token;
						$cookies.put('vendorfuel-tenant-token', tenant.tokens.jwt);
						tenant.authed = true;
					}
					return response;
				}, function errorCallback(response) {
					if (response.xhrStatus === 'error') {
						service.login_errors.error = 'Invalid VendorFuel API URL.';
					}

					if (Object.keys(response.data.errors).length > 0) {
						Debug.error('Login errors: ', response.data.errors);
						service.login_errors = response.data.errors;
					}
					return response;
				});
				return promise;
			},
			LoginErrors: function () {
				return service.login_errors;
			},
			Logout: function () {
				tenant.name = null;
				$cookies.remove('vendorfuel-tenant-name');
				tenant.tokens.jwt = null;
				$cookies.remove('vendorfuel-tenant-token');
				tenant.authed = false;
			},
			Authed: function () {
				const token = $cookies.get('vendorfuel-tenant-token');
				if (token) {
					tenant.authed = true;
				} else {
					tenant.authed = false;
				}
				return tenant.authed;
			},
		};

		const tenant = {
			authed: false,
			name: '',
			tokens: {
				a: '',
				b: '',
				jwt: '',
			},
		};

		return service;
	}
}());

