<?php

class Vendorfuel_Initialize {

	protected $utilities;

	public function __construct( $u ) {
		$this->utilities = $u;
	}

	private function settings() {
		//verify plugin settings exist. if they do not, create them
		$settings          = array(
			'api_url'            => 'https://api.vendorfuel.com/production',
			'api_key'            => '',
			'bootstrap'          => 1,
			'product_slug'       => 'products',
			'cat_slug'           => 'categories',
			'requireAddress'     => 0,
			'debug'              => 0,
			'redirectToCart'     => 0,
			'pagination'         => 0,
			'enableRecyclable'   => 0,
			'checkout'           => array(
				'company_name_option'   => 0,
				'purchase_order_option' => 0,
				'issuing_office_option' => 0,
				'cost_center_option'    => 0,
				'attention_option'      => 0,
				'notes_option'          => 0,
			),
		);
		$existing_settings = (array) get_option( 'vendorfuel-general-settings' );
		if ( ! $existing_settings ) { //general settings
			add_option( 'vendorfuel-general-settings', json_decode( wp_json_encode( $settings ), false ) );
		} else {
			if ( isset( $existing_settings['apiKey'] ) ) {
				$settings['api_key'] = $existing_settings['apiKey'];
                unset($existing_settings['apiKey']);
			}

			if ( isset( $existing_settings['apiUrl'] ) ) {
				$settings['api_url'] = $existing_settings['apiUrl'];
                unset($existing_settings['apiUrl']);
			}

			if ( isset( $existing_settings['productSlug'] ) ) {
				$settings['product_slug'] = $existing_settings['productSlug'];
                unset($existing_settings['productSlug']);
			}

			if ( isset( $existing_settings['catSlug'] ) ) {
				$settings['cat_slug'] = $existing_settings['catSlug'];
				unset($existing_settings['catSlug']);
			}

			update_option( 'vendorfuel-general-settings', json_decode( wp_json_encode( array_merge( $settings, (array) $existing_settings ) ), false ) );
		}

		if ( ! get_option( 'vendorfuel-conversions-settings' ) ) { //conversions settings
			$settings = array(
				'google' => array(
					'enabled'  => false,
					'id'       => '',
					'label'    => '',
					'tracking' => '',
				),
			);
			add_option( 'vendorfuel-conversions-settings', json_decode( wp_json_encode( $settings ), false ) );
		}
		$store_options          = array(
			'name'    => '',
			'url'     => '',
			'options' => json_decode(
				wp_json_encode(
					array(
						'Inventory System'                => false,
						'Admin Two Factor Authentication' => false,
						'Verify Users'                    => false,
						'Disable Guests'                  => false,
						'Disable RMA'                     => false,
						'Disable Billing Address'         => false,
						'Force Role'                      => false,
						'Credit Line Default On'          => false,
					)
				),
				true
			),
		);
		$existing_store_options = get_option( 'vendorfuel-store-options' );
		if ( ! $existing_store_options ) { //conversions settings
			add_option( 'vendorfuel-store-options', json_decode( wp_json_encode( $store_options ), false ) );
		} else {
			update_option( 'vendorfuel-store-options', json_decode( wp_json_encode( array_merge( $store_options, (array) $existing_store_options ) ), false ) );
		}
		if ( ! get_option( 'vendorfuel-image-settings' ) ) { //image settings
			$settings = array(
				'storage'        => array(
					'repository' => 0,
					'api_key'    => '',
				),
				'thumbnail_size' => array(
					'w'    => 120,
					'h'    => 90,
					'crop' => true,
				),
				'medium_size'    => array(
					'w' => 480,
					'h' => 396,
				),
				'large_size'     => array(
					'w' => 680,
					'h' => 300,
				),
			);
			add_option( 'vendorfuel-image-settings', json_decode( wp_json_encode( $settings ), false ) );
		}
		if ( ! get_option( 'vendorfuel-anguledit-settings' ) ) {
			$settings = array(
				'devBranchActive'  => false,
				'template_map'     => array(),
				'editHistoryCount' => 5,
			);
			add_option( 'vendorfuel-anguledit-settings', json_decode( wp_json_encode( $settings ), false ) );
		}

		if ( ! get_option( 'vendorfuel-analytics-settings' ) ) {
			$settings = array(
				'UA'  => [
					'enabled' => false,
					'id' => ''
				],
				'verification' => [
					'code' => ''
				],
				'AW' => [
					'enabled' => false,
					'id' => ''
				],
				'conversions' => [
					'phone' => [
						'enabled' => false,
						'number' => '',
						'tag' => ''
					],
					'purchase' => [
						'enabled' => false,
						'tag' => ''
					]
				]
			);	

			add_option( 'vendorfuel-analytics-settings', json_decode( wp_json_encode( $settings ), false ) );
		} 
		
	}
	private function pages() {
		//add required plugin pages
		$anguledit_options = get_option( 'vendorfuel-anguledit-settings' );

		$page_arr     = array( //TODO: move this into a template map creator on the AngulEdit view. This template map will allow pages to be accessed in the VendorFuel app as individual objects with parameters like url.
			array(
				'title'    => 'Account',
				'template' => 'account',
			),
			array(
				'title'    => 'Addresses',
				'template' => 'addresses',
			),
			array(
				'title'    => 'Cart',
				'template' => 'cart',
			),
			array(
				'title'    => 'Catalog',
				'template' => 'catalog',
			),
			array(
				'title'    => 'Checkout',
				'template' => 'checkout',
			),
			array(
				'title'    => 'Favorites',
				'template' => 'favorites',
			),
			array(
				'title'    => 'Forgot Password',
				'template' => 'forgot-password',
			),
			array(
				'title'    => 'Group Account',
				'template' => 'group-account',
			),
			array(
				'title'    => 'Group Orders',
				'template' => 'group-orders',
			),
			array(
				'title'    => 'Login',
				'template' => 'login',
			),
			array(
				'title'    => 'Order by SKU',
				'template' => 'order-by-sku',
			),
			array(
				'title'    => 'Orders',
				'template' => 'orders',
			),
			array(
				'title'    => 'Punchout Return',
				'template' => 'punchout-return',
			),
			array(
				'title'    => 'Register',
				'template' => 'register',
			),
			array(
				'title'    => 'Reset Password',
				'template' => 'reset-password',
			),
			array(
				'title'    => 'Saved Cart',
				'template' => 'saved-cart',
			),
			array(
				'title'    => 'Saved Carts',
				'template' => 'saved-carts',
			),
			array(
				'title'    => 'View Order',
				'template' => 'view-order',
			),
			array(
				'title'    => 'Welcome',
				'template' => 'welcome',
			),
		);
		$template_map = array();
		foreach ( $page_arr as $page ) {
			$temp_page = get_page_by_title( $page['title'] );
			if ( ! $temp_page && '' !== $page['title'] ) {
				$slug                  = strtolower( str_replace( ' ', '-', $page['title'] ) );
				$new_page              = array(
					'post_title'   => wp_strip_all_tags( $page['title'] ),
					'post_content' => '[vf-template name="' . $page['template'] . '"]',
					'post_type'    => 'page',
					'post_status'  => 'publish',
					'post_slug'    => $slug,
				);
				$id                    = wp_insert_post( $new_page );
				$page_url              = get_permalink( $id );
				$template_map[ $slug ] = array(
					'title'    => $page['title'],
					'template' => $page['template'],
					'url'      => $page_url,
					'id'       => $id,
				);
			} elseif ( null !== $temp_page ) {
				$temp_page             = $temp_page->to_array();
				$id                    = $temp_page['ID'];
				$slug                  = strtolower( str_replace( ' ', '-', $page['title'] ) );
				$page_url              = get_permalink( $id );
				$template_map[ $slug ] = array(
					'title'    => $page['title'],
					'template' => $page['template'],
					'url'      => $page_url,
					'id'       => $id,
				);
			}
		}
		if ( ! empty( $template_map ) ) {
			if ( $anguledit_options->template_map ) {
				$template_map = array_merge( $template_map, json_decode( wp_json_encode( $anguledit_options->template_map ), true ) );
			}
			$settings = array(
				'devBranchActive' => false,
				'template_map'    => json_decode( wp_json_encode( $template_map ), true ),
			);
			update_option( 'vendorfuel-anguledit-settings', json_decode( wp_json_encode( $settings ), false ) );
		}
	}

	public function initialize() {
		//initialize the plugin
		$this->settings(); //verify and initialize plugin settings
		$this->pages(); //create required pages
	}
}
