/**
 * Alert List Component
 *
 * @namespace Components
 */
( function() {
	'use strict';

	const template = `
        <p ng-repeat="alert in $ctrl.list track by $index"
            ng-if="alert.msg"
            ng-class="['has-background', {
                'has-luminous-vivid-amber-background-color': alert.type === 'warning',
                'has-white-color has-vivid-red-background-color has-text-color': alert.type === 'danger',
                'has-pale-cyan-blue-background-color': alert.type === 'info',
                'has-light-green-cyan-background-color': alert.type === 'success',
                'has-very-light-gray-background-color':
                    alert.type !== 'warning' &&
                    alert.type !== 'danger' &&
                    alert.type !== 'info' &&
                    alert.type !== 'success',
            }]">
            {{ alert.msg }}
            <a class="alignright text-decoration-none" href="#"
                ng-click="$ctrl.close($event, $index)">
                <span ng-class="alert.type === 'danger' ? 'has-inline-color has-white-color' : 'has-inline-color has-black-color'">
                    <i class="bi bi-x" aria-label="Close"></i>
                </span>
            </a> 
        </p>
    `;

	angular
		.module( 'vfApp' )
		.component( 'alertList', {
			controller: AlertListController,
			template,
		} );

	AlertListController.$inject = [
		'Alerts',
	];

	/**
	 * @param {Object} Alerts VendorFuel service
	 */
	function AlertListController(
		Alerts,
	) {
		const vm = this;
		vm.close = close;
		vm.list = Alerts.list;

		/**
		 * @param {Object} event Click event
		 * @param {number} index List index
		 */
		function close( event, index ) {
			event.preventDefault();
			vm.list.splice( index, 1 );
		}
	}
}() );
