/**
 * Payfabric Return Component
 *
 * @namespace Components
 */
( function() {
	'use strict';

	angular
		.module( 'vfApp' )
		.component( 'payfabricReturn', {
			controller: ReturnController,
			templateUrl: 'payfabricReturn.html',
		} );

	ReturnController.$inject = [
		'$rootScope',
		'$timeout',
		'Alerts',
		'Cart',
		'Utils',
	];

	/**
	 * @namespace ReturnController
	 * @param {Object} $rootScope AngularJs service
	 * @param {Object} $timeout   AngularJS service.
	 * @param {Object} Alerts     VendorFuel service.
	 * @param {Object} Cart       VendorFuel service.
	 * @param {Object} Utils      VendorFuel service.
	 * @memberof Components
	 */
	function ReturnController(
		$rootScope,
		$timeout,
		Alerts,
		Cart,
		Utils,
	) {
		const vm = this;
		vm.isLoading = true;
		vm.loadingText = 'Submitting order, please wait...';
		vm.$onInit = $onInit;

		/**
		 */
		function $onInit() {
			const params = {
				trx_key: Utils.urlParser.param( 'TrxKey' ),
				order_id: Utils.urlParser.param( 'OrderID' ),
			};
			const code = Utils.urlParser.param( 'PayFabricErrorCode' );
			const msg = Utils.urlParser.param( 'ResponseMsg' );
			const returnUrl = `https://${ window.location.hostname }/cc-return/?OrderID=${ params.order_id }`;
			const orderId = params.order_id;

			if ( code ) {
				vm.loadingText = msg.replace( '/+/g', ' ' ) + ', please try again.';
				Cart.payfabricUrl( returnUrl, orderId )
					.then( function( resp ) {
						$rootScope.payfabric_url = resp;
						parent.document.getElementById( 'payfabricFrame' ).src = resp;
					} );
			} else {
				vm.loadingText = 'Submitting order, please wait and do not refresh page...';
				params.payment_method = 'payfabric';
				Cart.completeOrder( params )
					.then( function( resolve ) {
						if ( resolve.data.errors.length < 1 ) {
							disableNavWarning();
							if ( resolve.data.pending === 1 ) {
								Alerts.message( 'Order is pending approval.' );
								$timeout( function() {
									window.location.href = Utils.getPageUrl( 'welcome' );
								}, 3000 );
							} else {
								top.location.href = `${ Utils.getPageUrl( 'view-order' ) }?id=${ resolve.data.order_id }`;
							}
						}
					} )
					.catch( ( reject ) => {
						console.error( 'rejected', reject );
					} );
			}
		}

		/**
		 * Prevents the user from navigating away from the page without a warning.
		 */
		function disableNavWarning() {
			// Needs to disable on the window parent since the Payfabric return is within an iframe.
			window.parent.onbeforeunload = null;
		}
	}
}() );
