import { alertsService } from './services/alerts.service';
import { analyticsService } from './services/analytics.service';
import { authService } from './services/auth.service';
import { cartService } from './services/cart.service';
import { CheckoutService } from './services/checkout.service';
import { debugService } from './services/debug.service';
import { favoritesService } from './services/favorites.service';
import { groupService } from './services/group.service';
import { localizedService } from './services/localized.service';
import { punchoutService } from './services/punchout.service';
import { userService } from './services/user.service';
import { utilsService } from './services/utils.service';
import { CollectionsFactory } from './factories/collections.factory';
import { interceptorFactory } from './factories/interceptor.factory';
import { productsFactory } from './factories/products.factory';
import { telFilter } from './filters/tel.filter';
declare const angular: ng.IAngularStatic;

export const PublicModule = angular
	.module( 'vfApp',
		[
			'ngResource',
			'LocalStorageModule',
			'ngCookies',
			'ngSanitize',
			'ui.mask',
		] )
	.config( [
		'$httpProvider',
		'localStorageServiceProvider',
		'$sceProvider',
		'$locationProvider',
		'uiMask.ConfigProvider',
		function(
			$httpProvider: ng.IHttpProvider,
			localStorageServiceProvider: ng.local.storage.ILocalStorageServiceProvider,
			$sceProvider: ng.ISCEProvider,
			$locationProvider: ng.ILocationProvider,
			uiMaskConfigProvider: any,
		) {
			$httpProvider.interceptors.push( 'vfInterceptor' );
			localStorageServiceProvider.setPrefix( 'vf' ).setStorageCookie( 0, '/', false ).setStorageType( 'sessionStorage' );
			$sceProvider.enabled( false );
			$locationProvider.html5Mode( {
				enabled: true,
				requireBase: false,
				rewriteLinks: false,
			} );
			uiMaskConfigProvider.maskDefinitions( { 2: /[2-9]/, 9: /\d/, A: /[a-zA-Z]/, '*': /[a-zA-Z0-9]/ } );
		} ] )
	.service( 'Alerts', alertsService )
	.service( 'Analytics', analyticsService )
	.service( 'Auth', authService )
	.service( 'Cart', cartService )
	.service( 'Checkout', CheckoutService )
	.service( 'Debug', debugService )
	.service( 'Favorites', favoritesService )
	.service( 'Group', groupService )
	.service( 'Localized', localizedService )
	.service( 'Punchout', punchoutService )
	.service( 'User', userService )
	.service( 'Utils', utilsService )
	.factory( 'Collections', CollectionsFactory )
	.factory( 'vfInterceptor', interceptorFactory )
	.factory( 'Products', productsFactory )
	.filter( 'tel', telFilter )
	.name;
